# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty, Tags
from .compat import policytypes
from .validators import (
    boolean,
    double,
    exactly_one,
    integer,
    integer_range,
    network_port,
    positive_integer,
    vpc_endpoint_type,
    vpn_pre_shared_key,
    vpn_tunnel_inside_cidr,
)

VALID_ELASTICINFERENCEACCELERATOR_TYPES = ("eia1.medium", "eia1.large", "eia1.xlarge")
VALID_CLIENTVPNENDPOINT_SELFSERVICEPORTAL = ("disabled", "enabled")
VALID_CLIENTVPNENDPOINT_VPNPORT = (443, 1194)


def validate_elasticinferenceaccelerator_type(elasticinferenceaccelerator_type):
    """Validate ElasticInferenceAccelerator for Instance"""

    if (
        elasticinferenceaccelerator_type not in VALID_ELASTICINFERENCEACCELERATOR_TYPES
    ):  # NOQA
        raise ValueError(
            "Elastic Inference Accelerator Type must be one of: %s"
            % ", ".join(VALID_ELASTICINFERENCEACCELERATOR_TYPES)  # NOQA
        )
    return elasticinferenceaccelerator_type


def validate_clientvpnendpoint_selfserviceportal(value):
    """Validate SelfServicePortal for ClientVpnEndpoint."""
    if value not in VALID_CLIENTVPNENDPOINT_SELFSERVICEPORTAL:
        raise ValueError(
            "ClientVpnEndpoint.SelfServicePortal must be one of: {}".format(
                ", ".join(VALID_CLIENTVPNENDPOINT_SELFSERVICEPORTAL)
            )
        )
    return value


def validate_clientvpnendpoint_vpnport(vpnport):
    """Validate VpnPort for ClientVpnEndpoint"""

    if vpnport not in VALID_CLIENTVPNENDPOINT_VPNPORT:
        raise ValueError(
            "ClientVpnEndpoint VpnPort must be one of: %s"
            % ", ".join(VALID_CLIENTVPNENDPOINT_VPNPORT)  # NOQA
        )
    return vpnport


class Tag(AWSProperty):
    props = {"Key": (str, True), "Value": (str, True)}

    def __init__(self, key=None, value=None, **kwargs):
        # provided for backward compatibility
        if key is not None:
            kwargs["Key"] = key
        if value is not None:
            kwargs["Value"] = value
        super().__init__(**kwargs)


class CarrierGateway(AWSObject):
    resource_type = "AWS::EC2::CarrierGateway"

    props = {
        "Tags": (Tags, False),
        "VpcId": (str, True),
    }


class CustomerGateway(AWSObject):
    resource_type = "AWS::EC2::CustomerGateway"

    props = {
        "BgpAsn": (integer, True),
        "IpAddress": (str, True),
        "Tags": ((Tags, list), False),
        "Type": (str, True),
    }


class DHCPOptions(AWSObject):
    resource_type = "AWS::EC2::DHCPOptions"

    props = {
        "DomainName": (str, False),
        "DomainNameServers": (list, False),
        "NetbiosNameServers": (list, False),
        "NetbiosNodeType": (integer, False),
        "NtpServers": (list, False),
        "Tags": ((Tags, list), False),
    }


class EgressOnlyInternetGateway(AWSObject):
    resource_type = "AWS::EC2::EgressOnlyInternetGateway"

    props = {
        "VpcId": (str, True),
    }


class EIP(AWSObject):
    resource_type = "AWS::EC2::EIP"

    props = {
        "InstanceId": (str, False),
        "Domain": (str, False),
        "PublicIpv4Pool": (str, False),
        "Tags": (Tags, False),
    }


class EIPAssociation(AWSObject):
    resource_type = "AWS::EC2::EIPAssociation"

    props = {
        "AllocationId": (str, False),
        "EIP": (str, False),
        "InstanceId": (str, False),
        "NetworkInterfaceId": (str, False),
        "PrivateIpAddress": (str, False),
    }


class EnclaveCertificateIamRoleAssociation(AWSObject):
    resource_type = "AWS::EC2::EnclaveCertificateIamRoleAssociation"

    props = {
        "CertificateArn": (str, True),
        "RoleArn": (str, True),
    }


class FlowLog(AWSObject):
    resource_type = "AWS::EC2::FlowLog"

    props = {
        "DeliverLogsPermissionArn": (str, False),
        "LogDestination": (str, False),
        "LogDestinationType": (str, False),
        "LogFormat": (str, False),
        "LogGroupName": (str, False),
        "MaxAggregationInterval": (integer, False),
        "ResourceId": (str, True),
        "ResourceType": (str, True),
        "Tags": (Tags, False),
        "TrafficType": (str, True),
    }


class NatGateway(AWSObject):
    resource_type = "AWS::EC2::NatGateway"

    props = {
        "AllocationId": (str, False),
        "ConnectivityType": (str, False),
        "SubnetId": (str, True),
        "Tags": ((Tags, list), False),
    }


class EBSBlockDevice(AWSProperty):
    props = {
        "DeleteOnTermination": (boolean, False),
        "Encrypted": (boolean, False),
        "KmsKeyId": (str, False),
        "Iops": (integer, False),  # Conditional
        "Throughput": (integer, False),  # Conditional
        "SnapshotId": (str, False),  # Conditional
        "VolumeSize": (integer, False),  # Conditional
        "VolumeType": (str, False),
    }


NO_DEVICE = {}


class BlockDeviceMapping(AWSProperty):
    props = {
        "DeviceName": (str, True),
        "Ebs": (EBSBlockDevice, False),  # Conditional
        "VirtualName": (str, False),  # Conditional
        "NoDevice": (dict, False),
    }


class LaunchTemplateBlockDeviceMapping(AWSProperty):
    props = {
        "DeviceName": (str, True),
        "Ebs": (EBSBlockDevice, False),  # Conditional
        "VirtualName": (str, False),  # Conditional
        "NoDevice": (str, False),
    }


class MountPoint(AWSProperty):
    props = {
        "Device": (str, True),
        "VolumeId": (str, True),
    }


class Placement(AWSProperty):
    props = {
        "Affinity": (str, False),
        "AvailabilityZone": (str, False),
        "GroupName": (str, False),
        "HostId": (str, False),
        "HostResourceGroupArn": (str, False),
        "PartitionNumber": (integer, False),
        "Tenancy": (str, False),
    }


class CpuOptions(AWSProperty):
    props = {
        "CoreCount": (integer, False),
        "ThreadsPerCore": (integer, False),
    }


class CreditSpecification(AWSProperty):
    props = {
        "CPUCredits": (str, False),
    }


class ElasticGpuSpecification(AWSProperty):
    props = {
        "Type": (str, True),
    }


class Ipv6Addresses(AWSHelperFn):
    def __init__(self, address):
        self.data = {
            "Ipv6Address": address,
        }


class LaunchTemplateSpecification(AWSProperty):
    props = {
        "LaunchTemplateId": (str, False),
        "LaunchTemplateName": (str, False),
        "Version": (str, True),
    }


class NetworkInsightsAnalysis(AWSObject):
    resource_type = "AWS::EC2::NetworkInsightsAnalysis"

    props = {
        "FilterInArns": ([str], False),
        "NetworkInsightsPathId": (str, True),
        "Tags": (Tags, False),
    }


class NetworkInsightsPath(AWSObject):
    resource_type = "AWS::EC2::NetworkInsightsPath"

    props = {
        "Destination": (str, True),
        "DestinationIp": (str, False),
        "DestinationPort": (integer, False),
        "Protocol": (str, True),
        "Source": (str, True),
        "SourceIp": (str, False),
        "Tags": (Tags, False),
    }


class PrivateIpAddressSpecification(AWSProperty):
    props = {
        "Primary": (boolean, True),
        "PrivateIpAddress": (str, True),
    }


class NetworkInterfaceProperty(AWSProperty):
    props = {
        "AssociateCarrierIpAddress": (boolean, False),
        "AssociatePublicIpAddress": (boolean, False),
        "DeleteOnTermination": (boolean, False),
        "Description": (str, False),
        "DeviceIndex": (integer, True),
        "GroupSet": ([str], False),
        "Ipv6AddressCount": (integer, False),
        "Ipv6Addresses": ([Ipv6Addresses], False),
        "NetworkCardIndex": (integer, False),
        "NetworkInterfaceId": (str, False),
        "PrivateIpAddress": (str, False),
        "PrivateIpAddresses": ([PrivateIpAddressSpecification], False),
        "SecondaryPrivateIpAddressCount": (integer, False),
        "SubnetId": (str, False),
    }


class AssociationParameters(AWSProperty):
    props = {
        "Key": (str, True),
        "Value": ([str], True),
    }


class SsmAssociations(AWSProperty):
    props = {
        "AssociationParameters": ([AssociationParameters], False),
        "DocumentName": (str, True),
    }


class GatewayRouteTableAssociation(AWSObject):
    resource_type = "AWS::EC2::GatewayRouteTableAssociation"

    props = {
        "GatewayId": (str, True),
        "RouteTableId": (str, True),
    }


class Host(AWSObject):
    resource_type = "AWS::EC2::Host"

    props = {
        "AutoPlacement": (str, False),
        "AvailabilityZone": (str, True),
        "HostRecovery": (str, False),
        "InstanceType": (str, True),
    }


class ElasticInferenceAccelerator(AWSProperty):
    props = {
        "Count": (integer, False),
        "Type": (validate_elasticinferenceaccelerator_type, True),
    }


class LicenseSpecification(AWSProperty):
    props = {
        "LicenseConfigurationArn": (str, True),
    }


class EnclaveOptions(AWSProperty):
    props = {
        "Enabled": (boolean, False),
    }


class HibernationOptions(AWSProperty):
    props = {
        "Configured": (boolean, False),
    }


class Instance(AWSObject):
    resource_type = "AWS::EC2::Instance"

    props = {
        "Affinity": (str, False),
        "AvailabilityZone": (str, False),
        "BlockDeviceMappings": (list, False),
        "CpuOptions": (CpuOptions, False),
        "CreditSpecification": (CreditSpecification, False),
        "DisableApiTermination": (boolean, False),
        "EbsOptimized": (boolean, False),
        "ElasticGpuSpecifications": ([ElasticGpuSpecification], False),
        "ElasticInferenceAccelerators": ([ElasticInferenceAccelerator], False),
        "EnclaveOptions": (EnclaveOptions, False),
        "HibernationOptions": (HibernationOptions, False),
        "HostId": (str, False),
        "HostResourceGroupArn": (str, False),
        "IamInstanceProfile": (str, False),
        "ImageId": (str, False),
        "InstanceInitiatedShutdownBehavior": (str, False),
        "InstanceType": (str, False),
        "Ipv6AddressCount": (integer, False),
        "Ipv6Addresses": ([Ipv6Addresses], False),
        "KernelId": (str, False),
        "KeyName": (str, False),
        "LaunchTemplate": (LaunchTemplateSpecification, False),
        "LicenseSpecifications": ([LicenseSpecification], False),
        "Monitoring": (boolean, False),
        "NetworkInterfaces": ([NetworkInterfaceProperty], False),
        "PlacementGroupName": (str, False),
        "PrivateIpAddress": (str, False),
        "RamdiskId": (str, False),
        "SecurityGroupIds": (list, False),
        "SecurityGroups": (list, False),
        "SsmAssociations": ([SsmAssociations], False),
        "SourceDestCheck": (boolean, False),
        "SubnetId": (str, False),
        "Tags": ((Tags, list), False),
        "Tenancy": (str, False),
        "UserData": (str, False),
        "Volumes": (list, False),
    }


class InternetGateway(AWSObject):
    resource_type = "AWS::EC2::InternetGateway"

    props = {
        "Tags": ((Tags, list), False),
    }


class NetworkAcl(AWSObject):
    resource_type = "AWS::EC2::NetworkAcl"

    props = {
        "Tags": ((Tags, list), False),
        "VpcId": (str, True),
    }


class ICMP(AWSProperty):
    props = {
        "Code": (integer, False),
        "Type": (integer, False),
    }


class PortRange(AWSProperty):
    props = {
        "From": (network_port, False),
        "To": (network_port, False),
    }


class NetworkAclEntry(AWSObject):
    resource_type = "AWS::EC2::NetworkAclEntry"

    props = {
        "CidrBlock": (str, False),
        "Egress": (boolean, False),
        "Icmp": (ICMP, False),  # Conditional
        "Ipv6CidrBlock": (str, False),
        "NetworkAclId": (str, True),
        "PortRange": (PortRange, False),  # Conditional
        "Protocol": (network_port, True),
        "RuleAction": (str, True),
        "RuleNumber": (integer_range(1, 32766), True),
    }

    def validate(self):
        conds = [
            "CidrBlock",
            "Ipv6CidrBlock",
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)


class NetworkInterface(AWSObject):
    resource_type = "AWS::EC2::NetworkInterface"

    props = {
        "Description": (str, False),
        "GroupSet": (list, False),
        "Ipv6AddressCount": (integer, False),
        "Ipv6Addresses": ([Ipv6Addresses], False),
        "PrivateIpAddress": (str, False),
        "PrivateIpAddresses": ([PrivateIpAddressSpecification], False),
        "SecondaryPrivateIpAddressCount": (integer, False),
        "SourceDestCheck": (boolean, False),
        "SubnetId": (str, True),
        "Tags": ((Tags, list), False),
    }


class NetworkInterfaceAttachment(AWSObject):
    resource_type = "AWS::EC2::NetworkInterfaceAttachment"

    props = {
        "DeleteOnTermination": (boolean, False),
        "DeviceIndex": (integer, True),
        "InstanceId": (str, True),
        "NetworkInterfaceId": (str, True),
    }


PERMISSION_INSTANCE_ATTACH = "INSTANCE-ATTACH"
PERMISSION_EIP_ASSOCIATE = "EIP-ASSOCIATE"


class NetworkInterfacePermission(AWSObject):
    resource_type = "AWS::EC2::NetworkInterfacePermission"

    props = {
        "AwsAccountId": (str, True),
        "NetworkInterfaceId": (str, True),
        "Permission": (str, True),
    }


class Entry(AWSProperty):
    props = {
        "Cidr": (str, True),
        "Description": (str, False),
    }


class PrefixList(AWSObject):
    resource_type = "AWS::EC2::PrefixList"

    props = {
        "AddressFamily": (str, True),
        "Entries": ([Entry], False),
        "MaxEntries": (integer, True),
        "PrefixListName": (str, True),
        "Tags": (Tags, False),
    }


class Route(AWSObject):
    resource_type = "AWS::EC2::Route"

    props = {
        "CarrierGatewayId": (str, False),
        "DestinationCidrBlock": (str, False),
        "DestinationIpv6CidrBlock": (str, False),
        "EgressOnlyInternetGatewayId": (str, False),
        "GatewayId": (str, False),
        "InstanceId": (str, False),
        "LocalGatewayId": (str, False),
        "NatGatewayId": (str, False),
        "NetworkInterfaceId": (str, False),
        "RouteTableId": (str, True),
        "TransitGatewayId": (str, False),
        "VpcEndpointId": (str, False),
        "VpcPeeringConnectionId": (str, False),
    }

    def validate(self):
        cidr_conds = [
            "DestinationCidrBlock",
            "DestinationIpv6CidrBlock",
        ]
        gateway_conds = [
            "CarrierGatewayId",
            "EgressOnlyInternetGatewayId",
            "GatewayId",
            "InstanceId",
            "LocalGatewayId",
            "NatGatewayId",
            "NetworkInterfaceId",
            "TransitGatewayId",
            "VpcEndpointId",
            "VpcPeeringConnectionId",
        ]
        exactly_one(self.__class__.__name__, self.properties, cidr_conds)
        exactly_one(self.__class__.__name__, self.properties, gateway_conds)


class RouteTable(AWSObject):
    resource_type = "AWS::EC2::RouteTable"

    props = {
        "Tags": ((Tags, list), False),
        "VpcId": (str, True),
    }


def check_ports(props):
    # IpProtocol is a required field but not all values allowed require
    # ToPort and FromPort. The ones that don't need these ports are:
    ports_optional = [
        "-1",  # all protocols
        "58",  # ICMPv6
    ]
    proto = props["IpProtocol"]

    if proto not in ports_optional:
        if not ("ToPort" in props and "FromPort" in props):
            raise ValueError("ToPort/FromPort must be specified for proto %s" % proto)


class SecurityGroupEgress(AWSObject):
    resource_type = "AWS::EC2::SecurityGroupEgress"

    props = {
        "CidrIp": (str, False),
        "CidrIpv6": (str, False),
        "Description": (str, False),
        "DestinationPrefixListId": (str, False),
        "DestinationSecurityGroupId": (str, False),
        "FromPort": (network_port, False),
        "GroupId": (str, True),
        "IpProtocol": (str, True),
        "ToPort": (network_port, False),
        #
        # Workaround for a bug in CloudFormation and EC2 where the
        # DestinationSecurityGroupId property is ignored causing
        # egress rules targeting a security group to be ignored.
        # Using SourceSecurityGroupId instead works fine even in
        # egress rules. AWS have known about this bug for a while.
        #
        "SourceSecurityGroupId": (str, False),
    }

    def validate(self):
        conds = [
            "CidrIp",
            "CidrIpv6",
            "DestinationPrefixListId",
            "DestinationSecurityGroupId",
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)
        check_ports(self.properties)


class SecurityGroupIngress(AWSObject):
    resource_type = "AWS::EC2::SecurityGroupIngress"

    props = {
        "CidrIp": (str, False),
        "CidrIpv6": (str, False),
        "Description": (str, False),
        "FromPort": (network_port, False),
        "GroupName": (str, False),
        "GroupId": (str, False),
        "IpProtocol": (str, True),
        "SourcePrefixListId": (str, False),
        "SourceSecurityGroupName": (str, False),
        "SourceSecurityGroupId": (str, False),
        "SourceSecurityGroupOwnerId": (str, False),
        "ToPort": (network_port, False),
    }

    def validate(self):
        conds = [
            "CidrIp",
            "CidrIpv6",
            "SourcePrefixListId",
            "SourceSecurityGroupName",
            "SourceSecurityGroupId",
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)
        check_ports(self.properties)


class SecurityGroupRule(AWSProperty):
    props = {
        "CidrIp": (str, False),
        "CidrIpv6": (str, False),
        "Description": (str, False),
        "DestinationPrefixListId": (str, False),
        "DestinationSecurityGroupId": (str, False),
        "FromPort": (network_port, False),
        "IpProtocol": (str, True),
        "SourcePrefixListId": (str, False),
        "SourceSecurityGroupId": (str, False),
        "SourceSecurityGroupName": (str, False),
        "SourceSecurityGroupOwnerId": (str, False),
        "ToPort": (network_port, False),
    }


class SecurityGroup(AWSObject):
    resource_type = "AWS::EC2::SecurityGroup"

    props = {
        "GroupName": (str, False),
        "GroupDescription": (str, True),
        "SecurityGroupEgress": (list, False),
        "SecurityGroupIngress": (list, False),
        "VpcId": (str, False),
        "Tags": ((Tags, list), False),
    }


class Subnet(AWSObject):
    resource_type = "AWS::EC2::Subnet"

    props = {
        "AssignIpv6AddressOnCreation": (boolean, False),
        "AvailabilityZone": (str, False),
        "CidrBlock": (str, True),
        "Ipv6CidrBlock": (str, False),
        "MapPublicIpOnLaunch": (boolean, False),
        "OutpostArn": (str, False),
        "Tags": ((Tags, list), False),
        "VpcId": (str, True),
    }

    def validate(self):
        if "Ipv6CidrBlock" in self.properties:
            if not self.properties.get("AssignIpv6AddressOnCreation"):
                raise ValueError(
                    "If Ipv6CidrBlock is present, "
                    "AssignIpv6AddressOnCreation must be set to True"
                )


class SubnetNetworkAclAssociation(AWSObject):
    resource_type = "AWS::EC2::SubnetNetworkAclAssociation"

    props = {
        "SubnetId": (str, True),
        "NetworkAclId": (str, True),
    }


class SubnetRouteTableAssociation(AWSObject):
    resource_type = "AWS::EC2::SubnetRouteTableAssociation"

    props = {
        "RouteTableId": (str, True),
        "SubnetId": (str, True),
    }


class Volume(AWSObject):
    resource_type = "AWS::EC2::Volume"

    props = {
        "AutoEnableIO": (boolean, False),
        "AvailabilityZone": (str, True),
        "Encrypted": (boolean, False),
        "Iops": (positive_integer, False),
        "KmsKeyId": (str, False),
        "MultiAttachEnabled": (boolean, False),
        "OutpostArn": (str, False),
        "Size": (positive_integer, False),
        "SnapshotId": (str, False),
        "Tags": ((Tags, list), False),
        "Throughput": (positive_integer, False),
        "VolumeType": (str, False),
    }


class VolumeAttachment(AWSObject):
    resource_type = "AWS::EC2::VolumeAttachment"

    props = {
        "Device": (str, True),
        "InstanceId": (str, True),
        "VolumeId": (str, True),
    }


def instance_tenancy(value):
    valid = ["default", "dedicated"]
    if value not in valid:
        raise ValueError("InstanceTenancy needs to be one of %r" % valid)
    return value


class VPC(AWSObject):
    resource_type = "AWS::EC2::VPC"

    props = {
        "CidrBlock": (str, True),
        "EnableDnsSupport": (boolean, False),
        "EnableDnsHostnames": (boolean, False),
        "InstanceTenancy": (instance_tenancy, False),
        "Tags": ((Tags, list), False),
    }


class VPCDHCPOptionsAssociation(AWSObject):
    resource_type = "AWS::EC2::VPCDHCPOptionsAssociation"

    props = {
        "DhcpOptionsId": (str, True),
        "VpcId": (str, True),
    }


class VPCEndpoint(AWSObject):
    resource_type = "AWS::EC2::VPCEndpoint"

    props = {
        "PolicyDocument": (policytypes, False),
        "PrivateDnsEnabled": (boolean, False),
        "RouteTableIds": ([str], False),
        "SecurityGroupIds": ([str], False),
        "ServiceName": (str, True),
        "SubnetIds": ([str], False),
        "VpcEndpointType": (vpc_endpoint_type, False),
        "VpcId": (str, True),
    }


class VPCEndpointConnectionNotification(AWSObject):
    resource_type = "AWS::EC2::VPCEndpointConnectionNotification"

    props = {
        "ConnectionEvents": ([str], True),
        "ConnectionNotificationArn": (str, True),
        "ServiceId": (str, False),
        "VPCEndpointId": (str, False),
    }


class VPCEndpointService(AWSObject):
    resource_type = "AWS::EC2::VPCEndpointService"

    props = {
        "AcceptanceRequired": (boolean, False),
        "GatewayLoadBalancerArns": ([str], False),
        "NetworkLoadBalancerArns": ([str], False),
    }


class VPCEndpointServicePermissions(AWSObject):
    resource_type = "AWS::EC2::VPCEndpointServicePermissions"

    props = {
        "AllowedPrincipals": ([str], False),
        "ServiceId": (str, True),
    }


class VPCGatewayAttachment(AWSObject):
    resource_type = "AWS::EC2::VPCGatewayAttachment"

    props = {
        "InternetGatewayId": (str, False),
        "VpcId": (str, True),
        "VpnGatewayId": (str, False),
    }


class VpnTunnelOptionsSpecification(AWSProperty):
    props = {
        "PreSharedKey": (vpn_pre_shared_key, False),
        "TunnelInsideCidr": (vpn_tunnel_inside_cidr, False),
    }


class VPNConnection(AWSObject):
    resource_type = "AWS::EC2::VPNConnection"

    props = {
        "CustomerGatewayId": (str, True),
        "StaticRoutesOnly": (boolean, False),
        "Tags": ((Tags, list), False),
        "TransitGatewayId": (str, False),
        "Type": (str, True),
        "VpnGatewayId": (str, False),
        "VpnTunnelOptionsSpecifications": ([VpnTunnelOptionsSpecification], False),
    }

    def validate(self):
        conds = [
            "VpnGatewayId",
            "TransitGatewayId",
        ]
        exactly_one(self.__class__.__name__, self.properties, conds)


class VPNConnectionRoute(AWSObject):
    resource_type = "AWS::EC2::VPNConnectionRoute"

    props = {
        "DestinationCidrBlock": (str, True),
        "VpnConnectionId": (str, True),
    }


class VPNGateway(AWSObject):
    resource_type = "AWS::EC2::VPNGateway"

    props = {
        "AmazonSideAsn": (positive_integer, False),
        "Type": (str, True),
        "Tags": ((Tags, list), False),
    }


class VPNGatewayRoutePropagation(AWSObject):
    resource_type = "AWS::EC2::VPNGatewayRoutePropagation"

    props = {
        "RouteTableIds": ([str], True),
        "VpnGatewayId": (str, True),
    }


class VPCPeeringConnection(AWSObject):
    resource_type = "AWS::EC2::VPCPeeringConnection"

    props = {
        "PeerVpcId": (str, True),
        "VpcId": (str, True),
        "Tags": ((Tags, list), False),
        "PeerRegion": (str, False),
        "PeerOwnerId": (str, False),
        "PeerRoleArn": (str, False),
    }


class Monitoring(AWSProperty):
    props = {
        "Enabled": (boolean, False),
    }


class NetworkInterfaces(AWSProperty):
    props = {
        "AssociatePublicIpAddress": (boolean, False),
        "DeleteOnTermination": (boolean, False),
        "Description": (str, False),
        "DeviceIndex": (integer, True),
        "Groups": ([str], False),
        "InterfaceType": (str, False),
        "Ipv6AddressCount": (integer, False),
        "Ipv6Addresses": ([Ipv6Addresses], False),
        "NetworkInterfaceId": (str, False),
        "PrivateIpAddresses": ([PrivateIpAddressSpecification], False),
        "SecondaryPrivateIpAddressCount": (integer, False),
        "SubnetId": (str, False),
    }


class SecurityGroups(AWSProperty):
    props = {
        "GroupId": (str, False),
    }


class IamInstanceProfile(AWSProperty):
    props = {
        "Arn": (str, False),
    }


class SpotFleetTagSpecification(AWSProperty):
    props = {
        "ResourceType": (str, True),
        "Tags": ((Tags, list), False),
    }


class LaunchSpecifications(AWSProperty):
    props = {
        "BlockDeviceMappings": ([BlockDeviceMapping], False),
        "EbsOptimized": (boolean, False),
        "IamInstanceProfile": (IamInstanceProfile, False),
        "ImageId": (str, True),
        "InstanceType": (str, True),
        "KernelId": (str, False),
        "KeyName": (str, False),
        "Monitoring": (Monitoring, False),
        "NetworkInterfaces": ([NetworkInterfaces], False),
        "Placement": (Placement, False),
        "RamdiskId": (str, False),
        "SecurityGroups": ([SecurityGroups], False),
        "SpotPrice": (str, False),
        "SubnetId": (str, False),
        "TagSpecifications": ([SpotFleetTagSpecification], False),
        "UserData": (str, False),
        "WeightedCapacity": (positive_integer, False),
    }


class LaunchTemplateOverrides(AWSProperty):
    props = {
        "AvailabilityZone": (str, False),
        "InstanceType": (str, False),
        "SpotPrice": (str, False),
        "SubnetId": (str, False),
        "WeightedCapacity": (double, False),
    }


class LaunchTemplateConfigs(AWSProperty):
    props = {
        "LaunchTemplateSpecification": (LaunchTemplateSpecification, True),
        "Overrides": ([LaunchTemplateOverrides], False),
    }


class ClassicLoadBalancer(AWSProperty):
    props = {"Name": (str, True)}


class ClassicLoadBalancersConfig(AWSProperty):
    props = {"ClassicLoadBalancers": ([ClassicLoadBalancer], True)}


class TargetGroup(AWSProperty):
    props = {"Arn": (str, True)}


class TargetGroupConfig(AWSProperty):
    props = {
        "TargetGroups": ([TargetGroup], True),
    }


class LoadBalancersConfig(AWSProperty):
    props = {
        "ClassicLoadBalancersConfig": ([ClassicLoadBalancersConfig], False),
        "TargetGroupsConfig": (TargetGroupConfig, False),
    }


class SpotCapacityRebalance(AWSProperty):
    props = {
        "ReplacementStrategy": (str, False),
    }


class SpotMaintenanceStrategies(AWSProperty):
    props = {
        "CapacityRebalance": (SpotCapacityRebalance, False),
    }


class SpotFleetRequestConfigData(AWSProperty):

    props = {
        "AllocationStrategy": (str, False),
        "ExcessCapacityTerminationPolicy": (str, False),
        "IamFleetRole": (str, True),
        "InstanceInterruptionBehavior": (str, False),
        "InstancePoolsToUseCount": (integer, False),
        "LaunchSpecifications": ([LaunchSpecifications], False),
        "LaunchTemplateConfigs": ([LaunchTemplateConfigs], False),
        "LoadBalancersConfig": (LoadBalancersConfig, False),
        "OnDemandAllocationStrategy": (str, False),
        "OnDemandMaxTotalPrice": (str, False),
        "OnDemandTargetCapacity": (integer, False),
        "ReplaceUnhealthyInstances": (boolean, False),
        "SpotMaintenanceStrategies": (SpotMaintenanceStrategies, False),
        "SpotMaxTotalPrice": (str, False),
        "SpotPrice": (str, False),
        "TargetCapacity": (positive_integer, True),
        "TerminateInstancesWithExpiration": (boolean, False),
        "Type": (str, False),
        "ValidFrom": (str, False),
        "ValidUntil": (str, False),
    }

    def validate(self):
        conds = ["LaunchSpecifications", "LaunchTemplateConfigs"]
        exactly_one(self.__class__.__name__, self.properties, conds)


class SpotFleet(AWSObject):
    resource_type = "AWS::EC2::SpotFleet"

    props = {
        "SpotFleetRequestConfigData": (SpotFleetRequestConfigData, True),
    }


class PlacementGroup(AWSObject):
    resource_type = "AWS::EC2::PlacementGroup"

    props = {
        "Strategy": (str, False),
    }


class SubnetCidrBlock(AWSObject):
    resource_type = "AWS::EC2::SubnetCidrBlock"

    props = {
        "Ipv6CidrBlock": (str, True),
        "SubnetId": (str, True),
    }


class VPCCidrBlock(AWSObject):
    resource_type = "AWS::EC2::VPCCidrBlock"

    props = {
        "AmazonProvidedIpv6CidrBlock": (boolean, False),
        "CidrBlock": (str, False),
        "Ipv6CidrBlock": (str, False),
        "Ipv6Pool": (str, False),
        "VpcId": (str, True),
    }


class TagSpecifications(AWSProperty):
    props = {"ResourceType": (str, False), "Tags": ((Tags, list), False)}


class CapacityReservationTarget(AWSProperty):
    props = {
        "CapacityReservationId": (str, False),
    }


class CapacityReservationSpecification(AWSProperty):
    props = {
        "CapacityReservationPreference": (str, False),
        "CapacityReservationTarget": (CapacityReservationTarget, False),
    }


class SpotOptions(AWSProperty):
    props = {
        "BlockDurationMinutes": (integer, False),
        "InstanceInterruptionBehavior": (str, False),
        "MaxPrice": (str, False),
        "SpotInstanceType": (str, False),
        "ValidUntil": (str, False),
    }


class InstanceMarketOptions(AWSProperty):
    props = {"MarketType": (str, False), "SpotOptions": (SpotOptions, False)}


class LaunchTemplateCreditSpecification(AWSProperty):
    props = {
        "CpuCredits": (str, False),
    }


class MetadataOptions(AWSProperty):
    props = {
        "HttpEndpoint": (str, False),
        "HttpPutResponseHopLimit": (integer, False),
        "HttpTokens": (str, False),
    }


class LaunchTemplateElasticInferenceAccelerator(AWSProperty):
    props = {
        "Count": (integer, False),
        "Type": (validate_elasticinferenceaccelerator_type, False),
    }


class LaunchTemplateData(AWSProperty):
    props = {
        "BlockDeviceMappings": ([LaunchTemplateBlockDeviceMapping], False),
        "CapacityReservationSpecification": (CapacityReservationSpecification, False),
        "CpuOptions": (CpuOptions, False),
        "CreditSpecification": (LaunchTemplateCreditSpecification, False),
        "DisableApiTermination": (boolean, False),
        "EbsOptimized": (boolean, False),
        "ElasticGpuSpecifications": ([ElasticGpuSpecification], False),
        "ElasticInferenceAccelerators": (
            [LaunchTemplateElasticInferenceAccelerator],
            False,
        ),  # NOQA
        "EnclaveOptions": (EnclaveOptions, False),
        "HibernationOptions": (HibernationOptions, False),
        "IamInstanceProfile": (IamInstanceProfile, False),
        "ImageId": (str, False),
        "InstanceInitiatedShutdownBehavior": (str, False),
        "InstanceMarketOptions": (InstanceMarketOptions, False),
        "InstanceType": (str, False),
        "KernelId": (str, False),
        "KeyName": (str, False),
        "LicenseSpecifications": ([LicenseSpecification], False),
        "MetadataOptions": (MetadataOptions, False),
        "Monitoring": (Monitoring, False),
        "NetworkInterfaces": ([NetworkInterfaces], False),
        "Placement": (Placement, False),
        "RamDiskId": (str, False),
        "SecurityGroups": (list, False),
        "SecurityGroupIds": (list, False),
        "TagSpecifications": ([TagSpecifications], False),
        "UserData": (str, False),
    }


class LaunchTemplate(AWSObject):
    resource_type = "AWS::EC2::LaunchTemplate"
    props = {
        "LaunchTemplateData": (LaunchTemplateData, False),
        "LaunchTemplateName": (str, False),
        "TagSpecifications": ([TagSpecifications], False),
    }


class TrafficMirrorFilter(AWSObject):
    resource_type = "AWS::EC2::TrafficMirrorFilter"

    props = {
        "Description": (str, False),
        "NetworkServices": ([str], False),
        "Tags": (Tags, False),
    }


class TrafficMirrorPortRange(AWSProperty):
    props = {
        "FromPort": (integer, True),
        "ToPort": (integer, True),
    }


class TrafficMirrorFilterRule(AWSObject):
    resource_type = "AWS::EC2::TrafficMirrorFilterRule"

    props = {
        "Description": (str, False),
        "DestinationCidrBlock": (str, True),
        "DestinationPortRange": (TrafficMirrorPortRange, False),
        "Protocol": (integer, False),
        "RuleAction": (str, True),
        "RuleNumber": (integer, True),
        "SourceCidrBlock": (str, True),
        "SourcePortRange": (TrafficMirrorPortRange, False),
        "TrafficDirection": (str, True),
        "TrafficMirrorFilterId": (str, True),
    }


class TrafficMirrorSession(AWSObject):
    resource_type = "AWS::EC2::TrafficMirrorSession"

    props = {
        "Description": (str, False),
        "NetworkInterfaceId": (str, True),
        "PacketLength": (integer, False),
        "SessionNumber": (integer, True),
        "Tags": (Tags, False),
        "TrafficMirrorFilterId": (str, True),
        "TrafficMirrorTargetId": (str, True),
        "VirtualNetworkId": (integer, False),
    }


class TrafficMirrorTarget(AWSObject):
    resource_type = "AWS::EC2::TrafficMirrorTarget"

    props = {
        "Description": (str, False),
        "NetworkInterfaceId": (str, False),
        "NetworkLoadBalancerArn": (str, False),
        "Tags": (Tags, False),
    }


class TransitGateway(AWSObject):
    resource_type = "AWS::EC2::TransitGateway"
    props = {
        "AmazonSideAsn": (integer, False),
        "AutoAcceptSharedAttachments": (str, False),
        "DefaultRouteTableAssociation": (str, False),
        "DefaultRouteTablePropagation": (str, False),
        "Description": (str, False),
        "DnsSupport": (str, False),
        "MulticastSupport": (str, False),
        "Tags": ((Tags, list), False),
        "VpnEcmpSupport": (str, False),
    }


class TransitGatewayAttachment(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayAttachment"
    props = {
        "SubnetIds": ([str], True),
        "Tags": ((Tags, list), False),
        "TransitGatewayId": (str, True),
        "VpcId": (str, True),
    }


class TransitGatewayConnectOptions(AWSProperty):
    props = {
        "Protocol": (str, False),
    }


class TransitGatewayConnect(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayConnect"

    props = {
        "Options": (TransitGatewayConnectOptions, True),
        "Tags": (Tags, False),
        "TransportTransitGatewayAttachmentId": (str, True),
    }


class TransitGatewayMulticastDomain(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayMulticastDomain"

    props = {
        "Options": (dict, False),
        "Tags": (Tags, False),
        "TransitGatewayId": (str, True),
    }


class TransitGatewayMulticastDomainAssociation(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayMulticastDomainAssociation"

    props = {
        "SubnetId": (str, True),
        "TransitGatewayAttachmentId": (str, True),
        "TransitGatewayMulticastDomainId": (str, True),
    }


class TransitGatewayMulticastGroupMember(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayMulticastGroupMember"

    props = {
        "GroupIpAddress": (str, True),
        "NetworkInterfaceId": (str, True),
        "TransitGatewayMulticastDomainId": (str, True),
    }


class TransitGatewayMulticastGroupSource(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayMulticastGroupSource"

    props = {
        "GroupIpAddress": (str, True),
        "NetworkInterfaceId": (str, True),
        "TransitGatewayMulticastDomainId": (str, True),
    }


class TransitGatewayPeeringAttachment(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayPeeringAttachment"

    props = {
        "PeerAccountId": (str, True),
        "PeerRegion": (str, True),
        "PeerTransitGatewayId": (str, True),
        "Tags": (Tags, False),
        "TransitGatewayId": (str, True),
    }


class TransitGatewayRoute(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayRoute"
    props = {
        "Blackhole": (boolean, False),
        "DestinationCidrBlock": (str, False),
        "TransitGatewayAttachmentId": (str, False),
        "TransitGatewayRouteTableId": (str, True),
    }


class TransitGatewayRouteTable(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayRouteTable"
    props = {
        "Tags": ((Tags, list), False),
        "TransitGatewayId": (str, True),
    }


class TransitGatewayRouteTableAssociation(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayRouteTableAssociation"
    props = {
        "TransitGatewayAttachmentId": (str, True),
        "TransitGatewayRouteTableId": (str, True),
    }


class TransitGatewayRouteTablePropagation(AWSObject):
    resource_type = "AWS::EC2::TransitGatewayRouteTablePropagation"
    props = {
        "TransitGatewayAttachmentId": (str, True),
        "TransitGatewayRouteTableId": (str, True),
    }


class FleetLaunchTemplateSpecificationRequest(AWSProperty):
    props = {
        "LaunchTemplateId": (str, False),
        "LaunchTemplateName": (str, False),
        "Version": (str, False),
    }


class FleetLaunchTemplateOverridesRequest(AWSProperty):
    props = {
        "AvailabilityZone": (str, False),
        "InstanceType": (str, False),
        "MaxPrice": (str, False),
        "Priority": (double, False),
        "SubnetId": (str, False),
        "WeightedCapacity": (double, False),
    }


class FleetLaunchTemplateConfigRequest(AWSProperty):
    props = {
        "LaunchTemplateSpecification": (FleetLaunchTemplateSpecificationRequest, False),
        "Overrides": ([FleetLaunchTemplateOverridesRequest], False),
    }


class OnDemandOptionsRequest(AWSProperty):
    props = {
        "AllocationStrategy": (str, False),
    }


class SpotOptionsRequest(AWSProperty):
    props = {
        "AllocationStrategy": (str, False),
        "InstanceInterruptionBehavior": (str, False),
        "InstancePoolsToUseCount": (integer, False),
    }


class TargetCapacitySpecificationRequest(AWSProperty):
    props = {
        "DefaultTargetCapacityType": (str, False),
        "OnDemandTargetCapacity": (integer, False),
        "SpotTargetCapacity": (integer, False),
        "TotalTargetCapacity": (integer, False),
    }


class EC2Fleet(AWSObject):
    resource_type = "AWS::EC2::EC2Fleet"
    props = {
        "ExcessCapacityTerminationPolicy": (str, False),
        "LaunchTemplateConfigs": ([FleetLaunchTemplateConfigRequest], True),
        "OnDemandOptions": (OnDemandOptionsRequest, False),
        "ReplaceUnhealthyInstances": (boolean, False),
        "SpotOptions": (SpotOptionsRequest, False),
        "TagSpecifications": ([TagSpecifications], False),
        "TargetCapacitySpecification": (TargetCapacitySpecificationRequest, False),
        "TerminateInstancesWithExpiration": (boolean, False),
        "Type": (str, False),
        "ValidFrom": (str, False),
        "ValidUntil": (str, False),
    }


class CapacityReservation(AWSObject):
    resource_type = "AWS::EC2::CapacityReservation"
    props = {
        "AvailabilityZone": (str, True),
        "EbsOptimized": (boolean, False),
        "EndDate": (str, False),
        "EndDateType": (str, False),
        "EphemeralStorage": (boolean, False),
        "InstanceCount": (integer, True),
        "InstanceMatchCriteria": (str, False),
        "InstancePlatform": (str, True),
        "InstanceType": (str, True),
        "TagSpecifications": ([TagSpecifications], False),
        "Tenancy": (str, False),
    }


class ClientVpnAuthorizationRule(AWSObject):
    resource_type = "AWS::EC2::ClientVpnAuthorizationRule"

    props = {
        "AccessGroupId": (str, False),
        "AuthorizeAllGroups": (boolean, False),
        "ClientVpnEndpointId": (str, True),
        "Description": (str, False),
        "TargetNetworkCidr": (str, True),
    }


class CertificateAuthenticationRequest(AWSProperty):
    props = {
        "ClientRootCertificateChainArn": (str, True),
    }


class DirectoryServiceAuthenticationRequest(AWSProperty):
    props = {"DirectoryId": (str, True)}


class FederatedAuthenticationRequest(AWSProperty):
    props = {"SAMLProviderArn": (str, True), "SelfServiceSAMLProviderArn": (str, False)}


class ClientAuthenticationRequest(AWSProperty):
    props = {
        "ActiveDirectory": (DirectoryServiceAuthenticationRequest, False),
        "FederatedAuthentication": (FederatedAuthenticationRequest, False),
        "MutualAuthentication": (CertificateAuthenticationRequest, False),
        "Type": (str, True),
    }


class ClientConnectOptions(AWSProperty):
    props = {"Enabled": (bool, True), "LambdaFunctionArn": (str, False)}


class ConnectionLogOptions(AWSProperty):
    props = {
        "CloudwatchLogGroup": (str, False),
        "CloudwatchLogStream": (str, False),
        "Enabled": (boolean, True),
    }


class ClientVpnEndpoint(AWSObject):
    resource_type = "AWS::EC2::ClientVpnEndpoint"

    props = {
        "AuthenticationOptions": ([ClientAuthenticationRequest], True),
        "ClientCidrBlock": (str, True),
        "ClientConnectOptions": (ClientConnectOptions, False),
        "ConnectionLogOptions": (ConnectionLogOptions, True),
        "Description": (str, False),
        "DnsServers": ([str], False),
        "SecurityGroupIds": ([str], False),
        "SelfServicePortal": (validate_clientvpnendpoint_selfserviceportal, False),
        "ServerCertificateArn": (str, True),
        "SplitTunnel": (boolean, False),
        "TagSpecifications": ([TagSpecifications], False),
        "TransportProtocol": (str, False),
        "VpcId": (str, False),
        "VpnPort": (validate_clientvpnendpoint_vpnport, False),
    }


class ClientVpnRoute(AWSObject):
    resource_type = "AWS::EC2::ClientVpnRoute"

    props = {
        "ClientVpnEndpointId": (str, True),
        "Description": (str, False),
        "DestinationCidrBlock": (str, True),
        "TargetVpcSubnetId": (str, True),
    }


class ClientVpnTargetNetworkAssociation(AWSObject):
    resource_type = "AWS::EC2::ClientVpnTargetNetworkAssociation"

    props = {
        "ClientVpnEndpointId": (str, True),
        "SubnetId": (str, True),
    }


class LocalGatewayRoute(AWSObject):
    resource_type = "AWS::EC2::LocalGatewayRoute"

    props = {
        "DestinationCidrBlock": (str, True),
        "LocalGatewayRouteTableId": (str, True),
        "LocalGatewayVirtualInterfaceGroupId": (str, True),
    }


class LocalGatewayRouteTableVPCAssociation(AWSObject):
    resource_type = "AWS::EC2::LocalGatewayRouteTableVPCAssociation"

    props = {
        "LocalGatewayRouteTableId": (str, True),
        "Tags": ((Tags, list), False),
        "VpcId": (str, True),
    }



from . import AWSObject, AWSProperty, Tags
from .validators import boolean, one_of

Bursting = "bursting"
Provisioned = "provisioned"


def throughput_mode_validator(mode):
    valid_modes = [Bursting, Provisioned]
    if mode not in valid_modes:
        raise ValueError(
            'ThroughputMode must be one of: "%s"' % (", ".join(valid_modes))
        )
    return mode


def provisioned_throughput_validator(throughput):
    if throughput < 0.0:
        raise ValueError(
            "ProvisionedThroughputInMibps must be greater than or equal to 0.0"
        )
    return throughput


class PosixUser(AWSProperty):
    props = {
        "Gid": (str, True),
        "SecondaryGids": ([str], False),
        "Uid": (str, True),
    }


class CreationInfo(AWSProperty):
    props = {
        "OwnerGid": (str, True),
        "OwnerUid": (str, True),
        "Permissions": (str, True),
    }


class RootDirectory(AWSProperty):
    props = {
        "CreationInfo": (CreationInfo, False),
        "Path": (str, False),
    }


class AccessPoint(AWSObject):
    resource_type = "AWS::EFS::AccessPoint"

    props = {
        "AccessPointTags": (Tags, False),
        "ClientToken": (str, False),
        "FileSystemId": (str, True),
        "PosixUser": (PosixUser, False),
        "RootDirectory": (RootDirectory, False),
    }


class LifecyclePolicy(AWSProperty):
    props = {
        "TransitionToIA": (str, True),
    }


class BackupPolicy(AWSProperty):
    props = {
        "Status": (str, True),
    }

    def validate(self):
        conds = ["DISABLED", "DISABLING", "ENABLED", "ENABLING"]
        one_of(self.__class__.__name__, self.properties, "Status", conds)


class FileSystem(AWSObject):
    resource_type = "AWS::EFS::FileSystem"

    props = {
        "AvailabilityZoneName": (str, False),
        "BackupPolicy": (BackupPolicy, False),
        "Encrypted": (boolean, False),
        "FileSystemPolicy": (dict, False),
        "FileSystemTags": (Tags, False),
        "KmsKeyId": (str, False),
        "LifecyclePolicies": ([LifecyclePolicy], False),
        "PerformanceMode": (str, False),
        "ProvisionedThroughputInMibps": (float, False),
        "ThroughputMode": (throughput_mode_validator, False),
    }


class MountTarget(AWSObject):
    resource_type = "AWS::EFS::MountTarget"

    props = {
        "FileSystemId": (str, True),
        "IpAddress": (str, False),
        "SecurityGroups": ([str], True),
        "SubnetId": (str, True),
    }



# Converted from ELBSample.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

import troposphere.ec2 as ec2
import troposphere.elasticloadbalancingv2 as elb
from troposphere import (
    Base64,
    FindInMap,
    GetAtt,
    Join,
    Output,
    Parameter,
    Ref,
    Template,
)


def AddAMI(template):
    template.add_mapping(
        "RegionMap",
        {
            "us-east-1": {"AMI": "ami-6411e20d"},
            "us-west-1": {"AMI": "ami-c9c7978c"},
            "us-west-2": {"AMI": "ami-fcff72cc"},
            "eu-west-1": {"AMI": "ami-37c2f643"},
            "ap-southeast-1": {"AMI": "ami-66f28c34"},
            "ap-northeast-1": {"AMI": "ami-9c03a89d"},
            "sa-east-1": {"AMI": "ami-a039e6bd"},
        },
    )


def main():
    template = Template()
    template.set_version("2010-09-09")

    template.set_description(
        "AWS CloudFormation Sample Template: NLB with 1 EC2 instance"
    )

    AddAMI(template)

    # Add the Parameters
    keyname_param = template.add_parameter(
        Parameter(
            "KeyName",
            Type="String",
            Default="mark",
            Description="Name of an existing EC2 KeyPair to "
            "enable SSH access to the instance",
        )
    )

    template.add_parameter(
        Parameter(
            "InstanceType",
            Type="String",
            Description="WebServer EC2 instance type",
            Default="m1.small",
            AllowedValues=[
                "t1.micro",
                "m1.small",
                "m1.medium",
                "m1.large",
                "m1.xlarge",
                "m2.xlarge",
                "m2.2xlarge",
                "m2.4xlarge",
                "c1.medium",
                "c1.xlarge",
                "cc1.4xlarge",
                "cc2.8xlarge",
                "cg1.4xlarge",
            ],
            ConstraintDescription="must be a valid EC2 instance type.",
        )
    )

    webport_param = template.add_parameter(
        Parameter(
            "WebServerPort",
            Type="String",
            Default="8888",
            Description="TCP/IP port of the web server",
        )
    )

    subnetA = template.add_parameter(
        Parameter("subnetA", Type="String", Default="subnet-096fd06d")
    )

    subnetB = template.add_parameter(
        Parameter("subnetB", Type="String", Default="subnet-1313ef4b")
    )

    VpcId = template.add_parameter(
        Parameter("VpcId", Type="String", Default="vpc-82c514e6")
    )

    # Define the instance security group
    instance_sg = template.add_resource(
        ec2.SecurityGroup(
            "InstanceSecurityGroup",
            GroupDescription="Enable SSH and HTTP access on the inbound port",
            SecurityGroupIngress=[
                ec2.SecurityGroupRule(
                    IpProtocol="tcp",
                    FromPort="22",
                    ToPort="22",
                    CidrIp="0.0.0.0/0",
                ),
                ec2.SecurityGroupRule(
                    IpProtocol="tcp",
                    FromPort=Ref(webport_param),
                    ToPort=Ref(webport_param),
                    CidrIp="0.0.0.0/0",
                ),
            ],
        )
    )

    eipA = template.add_resource(
        ec2.EIP(
            "eipA",
            Domain="vpc",
        )
    )

    eipB = template.add_resource(
        ec2.EIP(
            "eipB",
            Domain="vpc",
        )
    )

    # Add the web server instance
    WebInstance = template.add_resource(
        ec2.Instance(
            "WebInstance",
            SecurityGroups=[Ref(instance_sg)],
            KeyName=Ref(keyname_param),
            InstanceType=Ref("InstanceType"),
            ImageId=FindInMap("RegionMap", Ref("AWS::Region"), "AMI"),
            UserData=Base64(Ref(webport_param)),
        )
    )

    # Add the network LB
    NetworkLB = template.add_resource(
        elb.LoadBalancer(
            "NetworkLB",
            Name="NetworkLB",
            Scheme="internet-facing",
            SubnetMappings=[
                elb.SubnetMapping(
                    AllocationId=GetAtt(eipA, "AllocationId"), SubnetId=Ref(subnetA)
                ),
                elb.SubnetMapping(
                    AllocationId=GetAtt(eipB, "AllocationId"), SubnetId=Ref(subnetB)
                ),
            ],
            Type="network",
        )
    )

    TargetGroupWeb = template.add_resource(
        elb.TargetGroup(
            "TargetGroupWeb",
            HealthCheckIntervalSeconds="30",
            HealthCheckProtocol="HTTP",
            HealthCheckTimeoutSeconds="10",
            HealthyThresholdCount="4",
            Matcher=elb.Matcher(HttpCode="200"),
            Name="WebTarget",
            Port=Ref(webport_param),
            Protocol="HTTP",
            Targets=[
                elb.TargetDescription(Id=Ref(WebInstance), Port=Ref(webport_param))
            ],
            UnhealthyThresholdCount="3",
            VpcId=Ref(VpcId),
        )
    )

    template.add_resource(
        elb.Listener(
            "Listener",
            Port="80",
            Protocol="HTTP",
            LoadBalancerArn=Ref(NetworkLB),
            DefaultActions=[
                elb.Action(Type="forward", TargetGroupArn=Ref(TargetGroupWeb))
            ],
        )
    )

    template.add_output(
        Output(
            "URL",
            Description="URL of the sample website",
            Value=Join("", ["http://", GetAtt(NetworkLB, "DNSName")]),
        )
    )

    print(template.to_json())


if __name__ == "__main__":
    main()



import troposphere.ssm as ssm
from troposphere import Parameter, Ref, Template
from troposphere.constants import STRING

t = Template()
t.set_description("2012-09-09")

rhel_patch_group_name = t.add_parameter(
    Parameter(
        "RHELPatchGroupName",
        Type=STRING,
        Description="The value of the RHEL patch group tag "
        "that will have the baseline applied to them.",
    )
)

windows_patch_group_name = t.add_parameter(
    Parameter(
        "WindowsPatchGroupName",
        Type=STRING,
        Description="The value of the Windows patch group tag"
        "that will have baseline applied to them.",
    )
)

linux_filter_group = ssm.PatchFilterGroup(
    "LinuxGroup",
    PatchFilters=[
        ssm.PatchFilter("Linuxfilter", Key="CLASSIFICATION", Values=["Security"])
    ],
)

windows_filter_group = ssm.PatchFilterGroup(
    "windowsGroup",
    PatchFilters=[
        ssm.PatchFilter(
            "WindowsFilter", Key="CLASSIFICATION", Values=["SecurityUpdates"]
        )
    ],
)

linux_rule_group = ssm.RuleGroup(
    "LinuxRuleGroup",
    PatchRules=[
        ssm.Rule(
            "LinuxBaseRule",
            ApproveAfterDays=90,
            ComplianceLevel="CRITICAL",
            PatchFilterGroup=linux_filter_group,
        )
    ],
)

windows_rule_group = ssm.RuleGroup(
    "WindowsRuleGroup",
    PatchRules=[
        ssm.Rule(
            "WindowsBaseRule",
            ApproveAfterDays=90,
            ComplianceLevel="CRITICAL",
            PatchFilterGroup=windows_filter_group,
        )
    ],
)

t.add_resource(
    ssm.PatchBaseline(
        "RHELBASELINE",
        ApprovalRules=linux_rule_group,
        Description="Baseline containing all updates approved",
        Name=Ref(rhel_patch_group_name),
        OperatingSystem="REDHAT_ENTERPRISE_LINUX",
        PatchGroups=[Ref(rhel_patch_group_name)],
    )
)

t.add_resource(
    ssm.PatchBaseline(
        "WINDOWSBASELINE",
        ApprovalRules=windows_rule_group,
        Description="Baseline containing all updates approved",
        Name=Ref(windows_patch_group_name),
        OperatingSystem="WINDOWS",
        PatchGroups=[Ref(windows_patch_group_name)],
    )
)

t.add_resource(
    ssm.Parameter(
        "WindowsBaselineTimestampParameter",
        Name="/OS/WINDOWS",
        Type="String",
        Value="Windows TIMESTAMP_VALUE",
    )
)

t.add_resource(
    ssm.Parameter(
        "LinuxBaselineTimestampParameter",
        Name="/OS/REDHAT",
        Type="String",
        Value="Redhat TIME_STAMP_VALUE",
    )
)

print(t.to_json())



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean


class DimensionConfiguration(AWSProperty):
    props = {
        "DefaultDimensionValue": (str, True),
        "DimensionName": (str, True),
        "DimensionValueSource": (str, True),
    }


class CloudWatchDestination(AWSProperty):
    props = {
        "DimensionConfigurations": ([DimensionConfiguration], False),
    }


class KinesisFirehoseDestination(AWSProperty):
    props = {
        "DeliveryStreamARN": (str, True),
        "IAMRoleARN": (str, True),
    }


class EventDestination(AWSProperty):
    props = {
        "CloudWatchDestination": (CloudWatchDestination, False),
        "Enabled": (boolean, False),
        "KinesisFirehoseDestination": (KinesisFirehoseDestination, False),
        "MatchingEventTypes": ([str], True),
        "Name": (str, False),
    }


class ConfigurationSetEventDestination(AWSObject):
    resource_type = "AWS::SES::ConfigurationSetEventDestination"

    props = {
        "ConfigurationSetName": (str, True),
        "EventDestination": (EventDestination, True),
    }


class Topic(AWSProperty):
    props = {
        "DefaultSubscriptionStatus": (str, True),
        "Description": (str, False),
        "DisplayName": (str, True),
        "TopicName": (str, True),
    }


class ContactList(AWSObject):
    resource_type = "AWS::SES::ContactList"

    props = {
        "ContactListName": (str, False),
        "Description": (str, False),
        "Tags": (Tags, False),
        "Topics": ([Topic], False),
    }


class ConfigurationSet(AWSObject):
    resource_type = "AWS::SES::ConfigurationSet"

    props = {
        "Name": (str, False),
    }


class IpFilter(AWSProperty):
    props = {
        "Cidr": (str, True),
        "Policy": (str, True),
    }


class Filter(AWSProperty):
    props = {
        "IpFilter": (IpFilter, True),
        "Name": (str, False),
    }


class ReceiptFilter(AWSObject):
    resource_type = "AWS::SES::ReceiptFilter"

    props = {
        "Filter": (Filter, True),
    }


class ReceiptRuleSet(AWSObject):
    resource_type = "AWS::SES::ReceiptRuleSet"

    props = {
        "RuleSetName": (str, False),
    }


class AddHeaderAction(AWSProperty):
    props = {
        "HeaderName": (str, True),
        "HeaderValue": (str, True),
    }


class BounceAction(AWSProperty):
    props = {
        "Message": (str, True),
        "Sender": (str, True),
        "SmtpReplyCode": (str, True),
        "StatusCode": (str, False),
        "TopicArn": (str, False),
    }


class LambdaAction(AWSProperty):
    props = {
        "FunctionArn": (str, True),
        "InvocationType": (str, False),
        "TopicArn": (str, False),
    }


class S3Action(AWSProperty):
    props = {
        "BucketName": (str, True),
        "KmsKeyArn": (str, False),
        "ObjectKeyPrefix": (str, False),
        "TopicArn": (str, False),
    }


class SNSAction(AWSProperty):
    props = {
        "Encoding": (str, False),
        "TopicArn": (str, False),
    }


class StopAction(AWSProperty):
    props = {
        "Scope": (str, True),
        "TopicArn": (str, False),
    }


class WorkmailAction(AWSProperty):
    props = {
        "OrganizationArn": (str, True),
        "TopicArn": (str, False),
    }


class Action(AWSProperty):
    props = {
        "AddHeaderAction": (AddHeaderAction, False),
        "BounceAction": (BounceAction, False),
        "LambdaAction": (LambdaAction, False),
        "S3Action": (S3Action, False),
        "SNSAction": (SNSAction, False),
        "StopAction": (StopAction, False),
        "WorkmailAction": (WorkmailAction, False),
    }


class Rule(AWSProperty):
    props = {
        "Actions": ([Action], False),
        "Enabled": (boolean, False),
        "Name": (str, False),
        "Recipients": ([str], False),
        "ScanEnabled": (boolean, False),
        "TlsPolicy": (str, False),
    }


class ReceiptRule(AWSObject):
    resource_type = "AWS::SES::ReceiptRule"

    props = {
        "After": (str, False),
        "Rule": (Rule, True),
        "RuleSetName": (str, True),
    }


class EmailTemplate(AWSProperty):
    props = {
        "HtmlPart": (str, False),
        "SubjectPart": (str, False),
        "TemplateName": (str, False),
        "TextPart": (str, False),
    }


class Template(AWSObject):
    resource_type = "AWS::SES::Template"

    props = {
        "Template": (EmailTemplate, False),
    }



# Converted from CloudTrail example.
# http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html.

from troposphere import GetAtt, Join, Parameter, Ref, Template
from troposphere.cloudtrail import Trail
from troposphere.s3 import Bucket, BucketPolicy
from troposphere.sns import Subscription, Topic, TopicPolicy

t = Template()

t.set_version("2010-09-09")

OperatorEmail = t.add_parameter(
    Parameter(
        "OperatorEmail",
        Type="String",
        Description="Email address to notify when new logs are published.",
    )
)

S3Bucket = t.add_resource(Bucket("S3Bucket", DeletionPolicy="Retain"))

Topic = t.add_resource(
    Topic(
        "Topic",
        Subscription=[
            Subscription(
                Endpoint=Ref(OperatorEmail),
                Protocol="email",
            ),
        ],
    )
)

TopicPolicy = t.add_resource(
    TopicPolicy(
        "TopicPolicy",
        Topics=[Ref(Topic)],
        PolicyDocument={
            "Version": "2008-10-17",
            "Statement": [
                {
                    "Action": "SNS:Publish",
                    "Principal": {"Service": "cloudtrail.amazonaws.com"},
                    "Resource": "*",
                    "Effect": "Allow",
                    "Sid": "AWSCloudTrailSNSPolicy",
                }
            ],
        },
    )
)

BucketPolicy = t.add_resource(
    BucketPolicy(
        "BucketPolicy",
        PolicyDocument={
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Action": "s3:GetBucketAcl",
                    "Principal": {"Service": "cloudtrail.amazonaws.com"},
                    "Resource": Join("", ["arn:aws:s3:::", Ref(S3Bucket)]),
                    "Effect": "Allow",
                    "Sid": "AWSCloudTrailAclCheck",
                },
                {
                    "Action": "s3:PutObject",
                    "Principal": {"Service": "cloudtrail.amazonaws.com"},
                    "Resource": Join(
                        "",
                        [
                            "arn:aws:s3:::",
                            Ref(S3Bucket),
                            "/AWSLogs/",
                            Ref("AWS::AccountId"),
                            "/*",
                        ],
                    ),
                    "Effect": "Allow",
                    "Sid": "AWSCloudTrailWrite",
                    "Condition": {
                        "StringEquals": {"s3:x-amz-acl": "bucket-owner-full-control"}
                    },
                },
            ],
        },
        Bucket=Ref(S3Bucket),
    )
)

myTrail = t.add_resource(
    Trail(
        "myTrail",
        IsLogging=True,
        S3BucketName=Ref(S3Bucket),
        SnsTopicName=GetAtt(Topic, "TopicName"),
        DependsOn=["BucketPolicy", "TopicPolicy"],
    )
)

print(t.to_json())



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 5.3.0


from . import AWSObject, AWSProperty


class Admins(AWSProperty):
    props = {}


class DataLakeSettings(AWSObject):
    resource_type = "AWS::LakeFormation::DataLakeSettings"

    props = {
        "Admins": (Admins, False),
    }


class DataLakePrincipal(AWSProperty):
    props = {
        "DataLakePrincipalIdentifier": (str, False),
    }


class DatabaseResource(AWSProperty):
    props = {
        "Name": (str, False),
    }


class TableResource(AWSProperty):
    props = {
        "DatabaseName": (str, False),
        "Name": (str, False),
    }


class DataLocationResource(AWSProperty):
    props = {
        "S3Resource": (str, False),
    }


class ColumnWildcard(AWSProperty):
    props = {
        "ExcludedColumnNames": ([str], False),
    }


class TableWithColumnsResource(AWSProperty):
    props = {
        "ColumnNames": ([str], False),
        "ColumnWildcard": (ColumnWildcard, False),
        "DatabaseName": (str, False),
        "Name": (str, False),
    }


class Resource(AWSProperty):
    props = {
        "DatabaseResource": (DatabaseResource, False),
        "DataLocationResource": (DataLocationResource, False),
        "TableResource": (TableResource, False),
        "TableWithColumnsResource": (TableWithColumnsResource, False),
    }


class Permissions(AWSObject):
    resource_type = "AWS::LakeFormation::Permissions"

    props = {
        "DataLakePrincipal": (DataLakePrincipal, True),
        "Permissions": ([str], False),
        "PermissionsWithGrantOption": ([str], False),
        "Resource": (Resource, True),
    }



from troposphere import GetAtt, Join, Output, Ref, Template
from troposphere.apigateway import (
    ApiKey,
    ApiStage,
    Deployment,
    Integration,
    IntegrationResponse,
    Method,
    MethodResponse,
    QuotaSettings,
    Resource,
    RestApi,
    Stage,
    StageKey,
    ThrottleSettings,
    UsagePlan,
    UsagePlanKey,
)
from troposphere.awslambda import Code, Function
from troposphere.iam import Policy, Role

t = Template()

# Create the Api Gateway
rest_api = t.add_resource(RestApi("ExampleApi", Name="ExampleApi"))

# Create a Lambda function that will be mapped
code = [
    "var response = require('cfn-response');",
    "exports.handler = function(event, context) {",
    "   context.succeed('foobar!');",
    "   return 'foobar!';",
    "};",
]

# Create a role for the lambda function
t.add_resource(
    Role(
        "LambdaExecutionRole",
        Path="/",
        Policies=[
            Policy(
                PolicyName="root",
                PolicyDocument={
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Action": ["logs:*"],
                            "Resource": "arn:aws:logs:*:*:*",
                            "Effect": "Allow",
                        },
                        {"Action": ["lambda:*"], "Resource": "*", "Effect": "Allow"},
                    ],
                },
            )
        ],
        AssumeRolePolicyDocument={
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Action": ["sts:AssumeRole"],
                    "Effect": "Allow",
                    "Principal": {
                        "Service": ["lambda.amazonaws.com", "apigateway.amazonaws.com"]
                    },
                }
            ],
        },
    )
)

# Create the Lambda function
foobar_function = t.add_resource(
    Function(
        "FoobarFunction",
        Code=Code(ZipFile=Join("", code)),
        Handler="index.handler",
        Role=GetAtt("LambdaExecutionRole", "Arn"),
        Runtime="nodejs4.3",
    )
)

# Create a resource to map the lambda function to
resource = t.add_resource(
    Resource(
        "FoobarResource",
        RestApiId=Ref(rest_api),
        PathPart="foobar",
        ParentId=GetAtt("ExampleApi", "RootResourceId"),
    )
)

# Create a Lambda API method for the Lambda resource
method = t.add_resource(
    Method(
        "LambdaMethod",
        DependsOn="FoobarFunction",
        RestApiId=Ref(rest_api),
        AuthorizationType="NONE",
        ResourceId=Ref(resource),
        HttpMethod="GET",
        Integration=Integration(
            Credentials=GetAtt("LambdaExecutionRole", "Arn"),
            Type="AWS",
            IntegrationHttpMethod="POST",
            IntegrationResponses=[IntegrationResponse(StatusCode="200")],
            Uri=Join(
                "",
                [
                    "arn:aws:apigateway:eu-west-1:lambda:path/2015-03-31/functions/",
                    GetAtt("FoobarFunction", "Arn"),
                    "/invocations",
                ],
            ),
        ),
        MethodResponses=[MethodResponse("CatResponse", StatusCode="200")],
    )
)

# Create a deployment
stage_name = "v1"

deployment = t.add_resource(
    Deployment(
        "%sDeployment" % stage_name,
        DependsOn="LambdaMethod",
        RestApiId=Ref(rest_api),
    )
)

stage = t.add_resource(
    Stage(
        "%sStage" % stage_name,
        StageName=stage_name,
        RestApiId=Ref(rest_api),
        DeploymentId=Ref(deployment),
    )
)

key = t.add_resource(
    ApiKey(
        "ApiKey", StageKeys=[StageKey(RestApiId=Ref(rest_api), StageName=Ref(stage))]
    )
)

# Create an API usage plan
usagePlan = t.add_resource(
    UsagePlan(
        "ExampleUsagePlan",
        UsagePlanName="ExampleUsagePlan",
        Description="Example usage plan",
        Quota=QuotaSettings(Limit=50000, Period="MONTH"),
        Throttle=ThrottleSettings(BurstLimit=500, RateLimit=5000),
        ApiStages=[
            ApiStage(
                ApiId=Ref(rest_api),
                Stage=Ref(stage),
                Throttle=(
                    {
                        "/some/api/path/GET": ThrottleSettings(
                            BurstLimit=500, RateLimit=5000
                        )
                    }
                ),
            )
        ],
    )
)

# tie the usage plan and key together
usagePlanKey = t.add_resource(
    UsagePlanKey(
        "ExampleUsagePlanKey",
        KeyId=Ref(key),
        KeyType="API_KEY",
        UsagePlanId=Ref(usagePlan),
    )
)

# Add the deployment endpoint as an output
t.add_output(
    [
        Output(
            "ApiEndpoint",
            Value=Join(
                "",
                [
                    "https://",
                    Ref(rest_api),
                    ".execute-api.eu-west-1.amazonaws.com/",
                    stage_name,
                ],
            ),
            Description="Endpoint for this stage of the api",
        ),
        Output("ApiKey", Value=Ref(key), Description="API key"),
    ]
)

print(t.to_json())



# Copyright (c) 2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import (
    boolean,
    component_platforms,
    ebsinstanceblockdevicespecification_volume_type,
    imagepipeline_status,
    integer,
    schedule_pipelineexecutionstartcondition,
)


class Component(AWSObject):
    resource_type = "AWS::ImageBuilder::Component"

    props = {
        "ChangeDescription": (str, False),
        "Data": (str, False),
        "Description": (str, False),
        "KmsKeyId": (str, False),
        "Name": (str, True),
        "Platform": (component_platforms, True),
        "Tags": (dict, False),
        "Uri": (str, False),
        "Version": (str, True),
    }


class ComponentConfiguration(AWSProperty):
    props = {
        "ComponentArn": (str, False),
    }


class EbsInstanceBlockDeviceSpecification(AWSProperty):
    props = {
        "DeleteOnTermination": (boolean, False),
        "Encrypted": (boolean, False),
        "Iops": (integer, False),
        "KmsKeyId": (str, False),
        "SnapshotId": (str, False),
        "VolumeSize": (integer, False),
        "VolumeType": (ebsinstanceblockdevicespecification_volume_type, False),
    }


class InstanceBlockDeviceMapping(AWSProperty):
    props = {
        "DeviceName": (str, False),
        "Ebs": (EbsInstanceBlockDeviceSpecification, False),
        "NoDevice": (str, False),
        "VirtualName": (str, False),
    }


class InstanceConfiguration(AWSProperty):
    props = {
        "BlockDeviceMappings": ([InstanceBlockDeviceMapping], False),
        "Image": (str, False),
    }


class TargetContainerRepository(AWSProperty):
    props = {
        "RepositoryName": (str, False),
        "Service": (str, False),
    }


class ContainerRecipe(AWSObject):
    resource_type = "AWS::ImageBuilder::ContainerRecipe"

    props = {
        "Components": ([ComponentConfiguration], True),
        "ContainerType": (str, True),
        "Description": (str, False),
        "DockerfileTemplateData": (str, False),
        "DockerfileTemplateUri": (str, False),
        "ImageOsVersionOverride": (str, False),
        "InstanceConfiguration": (InstanceConfiguration, False),
        "KmsKeyId": (str, False),
        "Name": (str, True),
        "ParentImage": (str, True),
        "PlatformOverride": (str, False),
        "Tags": (Tags, False),
        "TargetRepository": (TargetContainerRepository, True),
        "Version": (str, True),
        "WorkingDirectory": (str, False),
    }


class LaunchTemplateConfiguration(AWSProperty):
    props = {
        "AccountId": (str, False),
        "LaunchTemplateId": (str, False),
        "SetDefaultVersion": (boolean, False),
    }


class Distribution(AWSProperty):
    props = {
        "AmiDistributionConfiguration": (dict, False),
        "ContainerDistributionConfiguration": (dict, False),
        "LaunchTemplateConfigurations": ([LaunchTemplateConfiguration], False),
        "LicenseConfigurationArns": ([str], False),
        "Region": (str, False),
    }


class DistributionConfiguration(AWSObject):
    resource_type = "AWS::ImageBuilder::DistributionConfiguration"

    props = {
        "Description": (str, False),
        "Distributions": ([Distribution], True),
        "Name": (str, True),
        "Tags": (dict, False),
    }


class ImageTestsConfiguration(AWSProperty):
    props = {
        "ImageTestsEnabled": (boolean, False),
        "TimeoutMinutes": (integer, False),
    }


class Image(AWSObject):
    resource_type = "AWS::ImageBuilder::Image"

    props = {
        "DistributionConfigurationArn": (str, False),
        "ImageRecipeArn": (str, True),
        "ImageTestsConfiguration": (ImageTestsConfiguration, True),
        "InfrastructureConfigurationArn": (str, True),
        "Tags": (dict, False),
    }


class S3Logs(AWSProperty):
    props = {
        "S3BucketName": (str, False),
        "S3KeyPrefix": (str, False),
    }


class Logging(AWSProperty):
    props = {
        "S3Logs": (S3Logs, False),
    }


class InfrastructureConfiguration(AWSObject):
    resource_type = "AWS::ImageBuilder::InfrastructureConfiguration"

    props = {
        "Description": (str, False),
        "InstanceProfileName": (str, True),
        "InstanceTypes": ([str], False),
        "KeyPair": (str, False),
        "Logging": (Logging, False),
        "Name": (str, True),
        "ResourceTags": (dict, False),
        "SecurityGroupIds": ([str], False),
        "SnsTopicArn": (str, False),
        "SubnetId": (str, False),
        "Tags": (dict, False),
        "TerminateInstanceOnFailure": (boolean, False),
    }


class ComponentConfiguration(AWSProperty):
    props = {
        "ComponentArn": (str, False),
    }


class ImageRecipe(AWSObject):
    resource_type = "AWS::ImageBuilder::ImageRecipe"

    props = {
        "BlockDeviceMappings": ([InstanceBlockDeviceMapping], False),
        "Components": ([ComponentConfiguration], True),
        "Description": (str, False),
        "Name": (str, True),
        "ParentImage": (str, True),
        "Tags": (dict, False),
        "Version": (str, True),
    }


class Schedule(AWSProperty):
    props = {
        "PipelineExecutionStartCondition": (
            schedule_pipelineexecutionstartcondition,
            False,
        ),  # NOQA
        "ScheduleExpression": (str, False),
    }


class ImagePipeline(AWSObject):
    resource_type = "AWS::ImageBuilder::ImagePipeline"

    props = {
        "Description": (str, False),
        "DistributionConfigurationArn": (str, False),
        "ImageRecipeArn": (str, True),
        "ImageTestsConfiguration": (ImageTestsConfiguration, False),
        "InfrastructureConfigurationArn": (str, True),
        "Name": (str, True),
        "Schedule": (Schedule, False),
        "Status": (imagepipeline_status, False),
        "Tags": (dict, False),
    }



# Copyright (c) 2012-2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer

VALID_CLUSTERINGKEYCOLUMN_ORDERBY = ("ASC", "DESC")
VALID_BILLINGMODE_MODE = ("ON_DEMAND", "PROVISIONED")


def validate_clusteringkeycolumn_orderby(clusteringkeycolumn_orderby):
    if clusteringkeycolumn_orderby not in VALID_CLUSTERINGKEYCOLUMN_ORDERBY:
        raise ValueError(
            "ClusteringKeyColumn OrderBy must be one of: %s"
            % ", ".join(VALID_CLUSTERINGKEYCOLUMN_ORDERBY)
        )
    return clusteringkeycolumn_orderby


def validate_billingmode_mode(billingmode_mode):
    if billingmode_mode not in VALID_BILLINGMODE_MODE:
        raise ValueError(
            "BillingMode Mode must be one of: %s" % ", ".join(VALID_BILLINGMODE_MODE)
        )
    return billingmode_mode


class Keyspace(AWSObject):
    resource_type = "AWS::Cassandra::Keyspace"

    props = {
        "KeyspaceName": (str, False),
        "Tags": (Tags, False),
    }


class Column(AWSProperty):
    props = {
        "ColumnName": (str, True),
        "ColumnType": (str, True),
    }


class ClusteringKeyColumn(AWSProperty):
    props = {
        "Column": (Column, True),
        "OrderBy": (validate_clusteringkeycolumn_orderby, False),
    }


class ProvisionedThroughput(AWSProperty):
    props = {
        "ReadCapacityUnits": (integer, True),
        "WriteCapacityUnits": (integer, True),
    }


class BillingMode(AWSProperty):
    props = {
        "Mode": (validate_billingmode_mode, True),
        "ProvisionedThroughput": (ProvisionedThroughput, False),
    }


class EncryptionSpecification(AWSProperty):
    props = {
        "EncryptionType": (str, True),
        "KmsKeyIdentifier": (str, False),
    }


class Table(AWSObject):
    resource_type = "AWS::Cassandra::Table"

    props = {
        "BillingMode": (BillingMode, False),
        "ClusteringKeyColumns": ([ClusteringKeyColumn], False),
        "EncryptionSpecification": (EncryptionSpecification, False),
        "KeyspaceName": (str, True),
        "PartitionKeyColumns": ([Column], True),
        "PointInTimeRecoveryEnabled": (boolean, False),
        "RegularColumns": ([Column], False),
        "TableName": (str, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .compat import policytypes
from .validators import boolean, integer, integer_range, positive_integer

VALID_VOLUME_TYPES = ("standard", "gp2", "io1")
VALID_TLS_SECURITY_POLICIES = (
    "Policy-Min-TLS-1-0-2019-07",
    "Policy-Min-TLS-1-2-2019-07",
)


def validate_volume_type(volume_type):
    """Validate VolumeType for ElasticsearchDomain"""
    if volume_type not in VALID_VOLUME_TYPES:
        raise ValueError(
            "Elasticsearch Domain VolumeType must be one of: %s"
            % ", ".join(VALID_VOLUME_TYPES)
        )
    return volume_type


def validate_tls_security_policy(tls_security_policy):
    """Validate TLS Security Policy for ElasticsearchDomain"""
    if tls_security_policy not in VALID_TLS_SECURITY_POLICIES:
        raise ValueError(
            "Minimum TLS Security Policy must be one of: %s"
            % ", ".join(VALID_TLS_SECURITY_POLICIES)
        )
    return tls_security_policy


class CognitoOptions(AWSProperty):
    props = {
        "Enabled": (boolean, False),
        "IdentityPoolId": (str, False),
        "RoleArn": (str, False),
        "UserPoolId": (str, False),
    }


class DomainEndpointOptions(AWSProperty):
    props = {
        "CustomEndpoint": (str, False),
        "CustomEndpointCertificateArn": (str, False),
        "CustomEndpointEnabled": (boolean, False),  # Conditional
        "EnforceHTTPS": (boolean, False),
        "TLSSecurityPolicy": (validate_tls_security_policy, False),
    }


class EBSOptions(AWSProperty):
    props = {
        "EBSEnabled": (boolean, False),
        "Iops": (positive_integer, False),
        "VolumeSize": (integer, False),
        "VolumeType": (validate_volume_type, False),
    }

    def validate(self):
        volume_type = self.properties.get("VolumeType")
        iops = self.properties.get("Iops")
        if volume_type == "io1" and not iops:
            raise ValueError("Must specify Iops if VolumeType is 'io1'.")


class ColdStorageOptions(AWSProperty):
    props = {
        "Enabled": (boolean, False),
    }


class ZoneAwarenessConfig(AWSProperty):
    props = {
        "AvailabilityZoneCount": (integer, False),
    }


class ElasticsearchClusterConfig(AWSProperty):
    props = {
        "ColdStorageOptions": (ColdStorageOptions, False),
        "DedicatedMasterCount": (integer, False),
        "DedicatedMasterEnabled": (boolean, False),
        "DedicatedMasterType": (str, False),
        "InstanceCount": (integer, False),
        "InstanceType": (str, False),
        "WarmCount": (integer, False),
        "WarmEnabled": (boolean, False),
        "WarmType": (str, False),
        "ZoneAwarenessConfig": (ZoneAwarenessConfig, False),
        "ZoneAwarenessEnabled": (boolean, False),
    }


class EncryptionAtRestOptions(AWSProperty):
    props = {
        "Enabled": (boolean, False),
        "KmsKeyId": (str, False),
    }


class NodeToNodeEncryptionOptions(AWSProperty):
    props = {
        "Enabled": (boolean, False),
    }


class SnapshotOptions(AWSProperty):
    props = {"AutomatedSnapshotStartHour": (integer_range(0, 23), False)}


class VPCOptions(AWSProperty):
    props = {"SecurityGroupIds": ([str], False), "SubnetIds": ([str], False)}


class MasterUserOptions(AWSProperty):
    props = {
        "MasterUserARN": (str, False),
        "MasterUserName": (str, False),
        "MasterUserPassword": (str, False),
    }


class AdvancedSecurityOptionsInput(AWSProperty):
    props = {
        "Enabled": (boolean, False),
        "InternalUserDatabaseEnabled": (boolean, False),
        "MasterUserOptions": (MasterUserOptions, False),
    }


class Domain(AWSObject):
    resource_type = "AWS::Elasticsearch::Domain"

    props = {
        "AccessPolicies": (policytypes, False),
        "AdvancedOptions": (dict, False),
        "AdvancedSecurityOptions": (AdvancedSecurityOptionsInput, False),
        "CognitoOptions": (CognitoOptions, False),
        "DomainName": (str, False),
        "DomainEndpointOptions": (DomainEndpointOptions, False),
        "EBSOptions": (EBSOptions, False),
        "ElasticsearchClusterConfig": (ElasticsearchClusterConfig, False),
        "ElasticsearchVersion": (str, False),
        "EncryptionAtRestOptions": (EncryptionAtRestOptions, False),
        "LogPublishingOptions": (dict, False),
        "NodeToNodeEncryptionOptions": (NodeToNodeEncryptionOptions, False),
        "SnapshotOptions": (SnapshotOptions, False),
        "Tags": ((Tags, list), False),
        "VPCOptions": (VPCOptions, False),
    }


# Backward compatibility
ElasticsearchDomain = Domain



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer

ONE_HOUR = "One_Hour"
THREE_HOURS = "Three_Hours"
SIX_HOURS = "Six_Hours"
TWELVE_HOURS = "Twelve_Hours"
TWENTYFOUR_HOURS = "TwentyFour_Hours"


class Scope(AWSProperty):
    props = {
        "ComplianceResourceId": (str, False),
        "ComplianceResourceTypes": ([str], False),
        "TagKey": (str, False),
        "TagValue": (str, False),
    }


class SourceDetails(AWSProperty):
    props = {
        "EventSource": (str, True),
        "MaximumExecutionFrequency": (str, False),
        "MessageType": (str, True),
    }

    def validate(self):
        valid_freqs = [
            ONE_HOUR,
            THREE_HOURS,
            SIX_HOURS,
            TWELVE_HOURS,
            TWENTYFOUR_HOURS,
        ]
        freq = self.properties.get("MaximumExecutionFrequency")
        if freq and freq not in valid_freqs:
            raise ValueError(
                "MaximumExecutionFrequency (given: %s) must be one of: %s"
                % (freq, ", ".join(valid_freqs))
            )


class Source(AWSProperty):
    props = {
        "Owner": (str, True),
        "SourceDetails": ([SourceDetails], False),
        "SourceIdentifier": (str, True),
    }


class ConfigRule(AWSObject):
    resource_type = "AWS::Config::ConfigRule"

    props = {
        "ConfigRuleName": (str, False),
        "Description": (str, False),
        "InputParameters": (dict, False),
        "MaximumExecutionFrequency": (str, False),
        "Scope": (Scope, False),
        "Source": (Source, True),
    }


class AggregationAuthorization(AWSObject):
    resource_type = "AWS::Config::AggregationAuthorization"

    props = {
        "AuthorizedAccountId": (str, True),
        "AuthorizedAwsRegion": (str, True),
    }


class OrganizationAggregationSource(AWSProperty):
    props = {
        "AllAwsRegions": (boolean, False),
        "AwsRegions": ([str], False),
        "RoleArn": (str, True),
    }


class AccountAggregationSources(AWSProperty):
    props = {
        "AccountIds": ([str], True),
        "AllAwsRegions": (boolean, False),
        "AwsRegions": ([str], False),
    }


class ConfigurationAggregator(AWSObject):
    resource_type = "AWS::Config::ConfigurationAggregator"

    props = {
        "AccountAggregationSources": ([AccountAggregationSources], False),
        "ConfigurationAggregatorName": (str, True),
        "OrganizationAggregationSource": (OrganizationAggregationSource, False),
    }


class RecordingGroup(AWSProperty):
    props = {
        "AllSupported": (boolean, False),
        "IncludeGlobalResourceTypes": (boolean, False),
        "ResourceTypes": ([str], False),
    }


class ConfigurationRecorder(AWSObject):
    resource_type = "AWS::Config::ConfigurationRecorder"

    props = {
        "Name": (str, False),
        "RecordingGroup": (RecordingGroup, False),
        "RoleARN": (str, True),
    }


class ConfigSnapshotDeliveryProperties(AWSProperty):
    props = {
        "DeliveryFrequency": (str, False),
    }


class DeliveryChannel(AWSObject):
    resource_type = "AWS::Config::DeliveryChannel"

    props = {
        "ConfigSnapshotDeliveryProperties": (ConfigSnapshotDeliveryProperties, False),
        "Name": (str, False),
        "S3BucketName": (str, True),
        "S3KeyPrefix": (str, False),
        "S3KmsKeyArn": (str, False),
        "SnsTopicARN": (str, False),
    }


class OrganizationCustomRuleMetadata(AWSProperty):
    props = {
        "Description": (str, False),
        "InputParameters": (str, False),
        "LambdaFunctionArn": (str, True),
        "MaximumExecutionFrequency": (str, False),
        "OrganizationConfigRuleTriggerTypes": ([str], True),
        "ResourceIdScope": (str, False),
        "ResourceTypesScope": ([str], False),
        "TagKeyScope": (str, False),
        "TagValueScope": (str, False),
    }


class OrganizationManagedRuleMetadata(AWSProperty):
    props = {
        "Description": (str, False),
        "InputParameters": (str, False),
        "MaximumExecutionFrequency": (str, False),
        "ResourceIdScope": (str, False),
        "ResourceTypesScope": ([str], False),
        "RuleIdentifier": (str, True),
        "TagKeyScope": (str, False),
        "TagValueScope": (str, False),
    }


class OrganizationConfigRule(AWSObject):
    resource_type = "AWS::Config::OrganizationConfigRule"

    props = {
        "ExcludedAccounts": ([str], False),
        "OrganizationConfigRuleName": (str, True),
        "OrganizationCustomRuleMetadata": (OrganizationCustomRuleMetadata, False),
        "OrganizationManagedRuleMetadata": (OrganizationManagedRuleMetadata, False),
    }


class SsmControls(AWSProperty):
    props = {
        "ConcurrentExecutionRatePercentage": (integer, False),
        "ErrorPercentage": (integer, False),
    }


class ExecutionControls(AWSProperty):
    props = {
        "SsmControls": (SsmControls, False),
    }


class RemediationConfiguration(AWSObject):
    resource_type = "AWS::Config::RemediationConfiguration"

    props = {
        "Automatic": (boolean, False),
        "ConfigRuleName": (str, True),
        "ExecutionControls": (ExecutionControls, False),
        "MaximumAutomaticAttempts": (integer, False),
        "Parameters": (dict, False),
        "ResourceType": (str, False),
        "RetryAttemptSeconds": (integer, False),
        "TargetId": (str, True),
        "TargetType": (str, True),
        "TargetVersion": (str, False),
    }


class ConformancePackInputParameter(AWSProperty):
    props = {
        "ParameterName": (str, True),
        "ParameterValue": (str, True),
    }


class ConformancePack(AWSObject):
    resource_type = "AWS::Config::ConformancePack"

    props = {
        "ConformancePackInputParameters": (
            [ConformancePackInputParameter],
            False,
        ),  # NOQA
        "ConformancePackName": (str, True),
        "DeliveryS3Bucket": (str, True),
        "DeliveryS3KeyPrefix": (str, False),
        "TemplateBody": (str, False),
        "TemplateS3Uri": (str, False),
    }


class OrganizationConformancePack(AWSObject):
    resource_type = "AWS::Config::OrganizationConformancePack"

    props = {
        "ConformancePackInputParameters": (
            [ConformancePackInputParameter],
            False,
        ),  # NOQA
        "DeliveryS3Bucket": (str, True),
        "DeliveryS3KeyPrefix": (str, False),
        "ExcludedAccounts": ([str], False),
        "OrganizationConformancePackName": (str, True),
        "TemplateBody": (str, False),
        "TemplateS3Uri": (str, False),
    }


class StoredQuery(AWSObject):
    resource_type = "AWS::Config::StoredQuery"

    props = {
        "QueryDescription": (str, False),
        "QueryExpression": (str, True),
        "QueryName": (str, True),
        "Tags": (Tags, False),
    }



# Copyright (c) 2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, canary_runtime_version, integer


class VPCConfig(AWSProperty):
    props = {
        "SecurityGroupIds": ([str], True),
        "SubnetIds": ([str], True),
        "VpcId": (str, False),
    }


class Schedule(AWSProperty):
    props = {
        "DurationInSeconds": (str, False),
        "Expression": (str, True),
    }


class RunConfig(AWSProperty):
    props = {
        "MemoryInMB": (integer, False),
        "TimeoutInSeconds": (integer, True),
    }


class Code(AWSProperty):
    props = {
        "Handler": (str, False),
        "S3Bucket": (str, False),
        "S3Key": (str, False),
        "S3ObjectVersion": (str, False),
        "Script": (str, False),
    }


class Canary(AWSObject):
    resource_type = "AWS::Synthetics::Canary"

    props = {
        "ArtifactS3Location": (str, True),
        "Code": (Code, True),
        "ExecutionRoleArn": (str, True),
        "FailureRetentionPeriod": (integer, False),
        "Name": (str, True),
        "RunConfig": (RunConfig, False),
        "RuntimeVersion": (canary_runtime_version, True),
        "Schedule": (Schedule, True),
        "StartCanaryAfterCreation": (boolean, True),
        "SuccessRetentionPeriod": (integer, False),
        "Tags": (Tags, False),
        "VPCConfig": (VPCConfig, False),
    }



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty, BaseAWSObject, Tags, encode_to_dict
from .validators import boolean, check_required, encoding, integer


class ModuleDefaultVersion(AWSObject):
    resource_type = "AWS::CloudFormation::ModuleDefaultVersion"

    props = {
        "Arn": (str, False),
        "ModuleName": (str, False),
        "VersionId": (str, False),
    }


class ModuleVersion(AWSObject):
    resource_type = "AWS::CloudFormation::ModuleVersion"

    props = {
        "ModuleName": (str, True),
        "ModulePackage": (str, True),
    }


class PublicTypeVersion(AWSObject):
    resource_type = "AWS::CloudFormation::PublicTypeVersion"

    props = {
        "Arn": (str, False),
        "LogDeliveryBucket": (str, False),
        "PublicVersionNumber": (str, False),
        "Type": (str, False),
        "TypeName": (str, False),
    }


class Publisher(AWSObject):
    resource_type = "AWS::CloudFormation::Publisher"

    props = {
        "AcceptTermsAndConditions": (boolean, True),
        "ConnectionArn": (str, False),
    }


class ResourceDefaultVersion(AWSObject):
    resource_type = "AWS::CloudFormation::ResourceDefaultVersion"

    props = {
        "TypeName": (str, False),
        "TypeVersionArn": (str, False),
        "VersionId": (str, False),
    }


class LoggingConfig(AWSProperty):
    props = {
        "LogGroupName": (str, False),
        "LogRoleArn": (str, False),
    }


class ResourceVersion(AWSObject):
    resource_type = "AWS::CloudFormation::ResourceVersion"

    props = {
        "ExecutionRoleArn": (str, False),
        "LoggingConfig": (LoggingConfig, False),
        "SchemaHandlerPackage": (str, True),
        "TypeName": (str, True),
    }


class Stack(AWSObject):
    resource_type = "AWS::CloudFormation::Stack"

    props = {
        "NotificationARNs": ([str], False),
        "Parameters": (dict, False),
        "Tags": ((Tags, list), False),
        "TemplateURL": (str, True),
        "TimeoutInMinutes": (integer, False),
    }


class AWSCustomObject(BaseAWSObject):
    dictname = "Properties"


class CustomResource(AWSCustomObject):
    resource_type = "AWS::CloudFormation::CustomResource"

    props = {"ServiceToken": (str, True)}


class AutoDeployment(AWSProperty):
    props = {
        "Enabled": (boolean, False),
        "RetainStacksOnAccountRemoval": (boolean, False),
    }


class OperationPreferences(AWSProperty):
    props = {
        "FailureToleranceCount": (integer, False),
        "FailureTolerancePercentage": (integer, False),
        "MaxConcurrentCount": (integer, False),
        "MaxConcurrentPercentage": (integer, False),
        "RegionConcurrencyType": (str, False),
        "RegionOrder": ([str], False),
    }


class Parameter(AWSProperty):
    props = {
        "ParameterKey": (str, True),
        "ParameterValue": (str, True),
    }


class DeploymentTargets(AWSProperty):
    props = {
        "Accounts": ([str], False),
        "OrganizationalUnitIds": ([str], False),
    }


class StackInstances(AWSProperty):
    props = {
        "DeploymentTargets": (DeploymentTargets, True),
        "ParameterOverrides": ([Parameter], False),
        "Regions": ([str], True),
    }


class StackSet(AWSObject):
    resource_type = "AWS::CloudFormation::StackSet"

    props = {
        "AdministrationRoleARN": (str, False),
        "AutoDeployment": (AutoDeployment, False),
        "CallAs": (str, False),
        "Capabilities": ([str], False),
        "Description": (str, False),
        "ExecutionRoleName": (str, False),
        "OperationPreferences": (OperationPreferences, False),
        "Parameters": ([Parameter], False),
        "PermissionModel": (str, False),
        "StackInstancesGroup": ([StackInstances], False),
        "StackSetName": (str, False),
        "Tags": (Tags, False),
        "TemplateBody": (str, False),
        "TemplateURL": (str, False),
    }


class WaitCondition(AWSObject):
    resource_type = "AWS::CloudFormation::WaitCondition"

    props = {
        "Count": (integer, False),
        "Handle": (str, False),
        "Timeout": (integer, False),
    }

    def validate(self):
        if "CreationPolicy" in self.resource:
            for k in self.props.keys():
                if k in self.properties:
                    raise ValueError(
                        "Property %s cannot be specified with CreationPolicy" % k
                    )
        else:
            required = ["Handle", "Timeout"]
            check_required(self.__class__.__name__, self.properties, required)


class WaitConditionHandle(AWSObject):
    resource_type = "AWS::CloudFormation::WaitConditionHandle"

    props = {}


class Metadata(AWSHelperFn):
    def __init__(self, *args):
        self.data = args

    def to_dict(self):
        t = []
        for i in self.data:
            t += list(encode_to_dict(i).items())
        return dict(t)


class InitFileContext(AWSHelperFn):
    def __init__(self, data):
        self.data = data


class InitFile(AWSProperty):
    props = {
        "content": (str, False),
        "mode": (str, False),
        "owner": (str, False),
        "encoding": (encoding, False),
        "group": (str, False),
        "source": (str, False),
        "authentication": (str, False),
        "context": (InitFileContext, False),
    }


class InitFiles(AWSHelperFn):
    def __init__(self, data):
        self.validate(data)
        self.data = data

    def validate(self, data):
        for k in data:
            if not isinstance(data[k], InitFile):
                raise ValueError("File '" + k + "' must be of type InitFile")


class InitService(AWSProperty):
    props = {
        "ensureRunning": (boolean, False),
        "enabled": (boolean, False),
        "files": (list, False),
        "packages": (dict, False),
        "sources": (list, False),
        "commands": (list, False),
    }


class InitServices(AWSHelperFn):
    def __init__(self, data):
        self.validate(data)
        self.data = data

    def validate(self, data):
        for k in data:
            if not isinstance(data[k], InitService):
                raise ValueError("Service '" + k + "' must be of type InitService")


class InitConfigSets(AWSHelperFn):
    def __init__(self, **kwargs):
        self.validate(dict(kwargs))
        self.data = kwargs

    def validate(self, config_sets):
        for k, v in config_sets.items():
            if not isinstance(v, list):
                raise ValueError("configSets values must be of type list")


class InitConfig(AWSProperty):
    props = {
        "groups": (dict, False),
        "users": (dict, False),
        "sources": (dict, False),
        "packages": (dict, False),
        "files": (dict, False),
        "commands": (dict, False),
        "services": (dict, False),
    }


def validate_authentication_type(auth_type):
    valid_types = ["S3", "basic"]
    if auth_type not in valid_types:
        raise ValueError("Type needs to be one of %r" % valid_types)
    return auth_type


class AuthenticationBlock(AWSProperty):
    props = {
        "accessKeyId": (str, False),
        "buckets": ([str], False),
        "password": (str, False),
        "secretKey": (str, False),
        "type": (validate_authentication_type, False),
        "uris": ([str], False),
        "username": (str, False),
        "roleName": (str, False),
    }


class Authentication(AWSHelperFn):
    def __init__(self, data):
        self.validate(data)
        self.data = {"AWS::CloudFormation::Authentication": data}

    def validate(self, data):
        for k, v in data.items():
            if not isinstance(v, AuthenticationBlock):
                raise ValueError(
                    "authentication block must be of type"
                    " cloudformation.AuthenticationBlock"
                )


class Init(AWSHelperFn):
    def __init__(self, data, **kwargs):
        self.validate(data, dict(kwargs))

        if isinstance(data, InitConfigSets):
            self.data = {
                "AWS::CloudFormation::Init": dict({"configSets": data}, **kwargs)
            }
        else:
            self.data = {"AWS::CloudFormation::Init": data}

    def validate(self, data, config_sets):
        if isinstance(data, InitConfigSets):
            for k, v in sorted(config_sets.items()):
                if not isinstance(v, InitConfig):
                    raise ValueError(
                        "init configs must of type ", "cloudformation.InitConfigSet"
                    )
        else:
            if "config" not in data:
                raise ValueError("config property is required")
            if not isinstance(data["config"], InitConfig):
                raise ValueError(
                    "config property must be of type cloudformation.InitConfig"
                )


class Macro(AWSCustomObject):
    resource_type = "AWS::CloudFormation::Macro"

    props = {
        "Description": (str, False),
        "FunctionName": (str, True),
        "LogGroupName": (str, False),
        "LogRoleARN": (str, False),
        "Name": (str, True),
    }


class TypeActivation(AWSObject):
    resource_type = "AWS::CloudFormation::TypeActivation"

    props = {
        "AutoUpdate": (boolean, False),
        "ExecutionRoleArn": (str, False),
        "LoggingConfig": (LoggingConfig, False),
        "MajorVersion": (str, False),
        "PublicTypeArn": (str, False),
        "PublisherId": (str, False),
        "Type": (str, False),
        "TypeName": (str, False),
        "TypeNameAlias": (str, False),
        "VersionBump": (str, False),
    }



# Converted from Route53_CNAME.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from troposphere import Join, Output, Parameter, Ref, Template
from troposphere.route53 import RecordSetType

t = Template()

t.set_description(
    "AWS CloudFormation Sample Template Route53_CNAME: Sample template "
    "showing how to create an Amazon Route 53 CNAME record.  It assumes that "
    "you already  have a Hosted Zone registered with Amazon Route 53. "
    "**WARNING** This template creates an Amazon EC2 instance. "
    "You will be billed for the AWS resources used if you create "
    "a stack from this template."
)

hostedzone = t.add_parameter(
    Parameter(
        "HostedZone",
        Description="The DNS name of an existing Amazon Route 53 hosted zone",
        Type="String",
    )
)

myDNSRecord = t.add_resource(
    RecordSetType(
        "myDNSRecord",
        HostedZoneName=Join("", [Ref(hostedzone), "."]),
        Comment="CNAME redirect to aws.amazon.com.",
        Name=Join(
            "",
            [Ref("AWS::StackName"), ".", Ref("AWS::Region"), ".", Ref(hostedzone), "."],
        ),
        Type="CNAME",
        TTL="900",
        ResourceRecords=["aws.amazon.com"],
    )
)


t.add_output(Output("DomainName", Value=Ref(myDNSRecord)))

print(t.to_json())



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 8.1.0


from . import AWSObject, AWSProperty


class Target(AWSProperty):
    props = {
        "TargetAddress": (str, False),
        "TargetType": (str, False),
    }


class NotificationRule(AWSObject):
    resource_type = "AWS::CodeStarNotifications::NotificationRule"

    props = {
        "DetailType": (str, True),
        "EventTypeIds": ([str], True),
        "Name": (str, True),
        "Resource": (str, True),
        "Status": (str, False),
        "Tags": (dict, False),
        "Targets": ([Target], True),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import (
    boolean,
    elb_name,
    integer,
    integer_range,
    network_port,
    positive_integer,
)


class AppCookieStickinessPolicy(AWSProperty):
    props = {
        "CookieName": (str, True),
        "PolicyName": (str, True),
    }


class HealthCheck(AWSProperty):
    props = {
        "HealthyThreshold": (integer_range(2, 10), True),
        "Interval": (positive_integer, True),
        "Target": (str, True),
        "Timeout": (positive_integer, True),
        "UnhealthyThreshold": (integer_range(2, 10), True),
    }


class LBCookieStickinessPolicy(AWSProperty):
    props = {
        "CookieExpirationPeriod": (str, False),
        "PolicyName": (str, False),
    }


class Listener(AWSProperty):
    props = {
        "InstancePort": (network_port, True),
        "InstanceProtocol": (str, False),
        "LoadBalancerPort": (network_port, True),
        "PolicyNames": (list, False),
        "Protocol": (str, True),
        "SSLCertificateId": (str, False),
    }


class Policy(AWSProperty):
    props = {
        "Attributes": ([dict], False),
        "InstancePorts": (list, False),
        "LoadBalancerPorts": (list, False),
        "PolicyName": (str, True),
        "PolicyType": (str, True),
    }


class ConnectionDrainingPolicy(AWSProperty):
    props = {"Enabled": (bool, True), "Timeout": (integer, False)}


class ConnectionSettings(AWSProperty):
    props = {
        "IdleTimeout": (integer, True),
    }


class AccessLoggingPolicy(AWSProperty):
    props = {
        "EmitInterval": (integer, False),
        "Enabled": (bool, True),
        "S3BucketName": (str, False),
        "S3BucketPrefix": (str, False),
    }


class LoadBalancer(AWSObject):
    resource_type = "AWS::ElasticLoadBalancing::LoadBalancer"

    props = {
        "AccessLoggingPolicy": (AccessLoggingPolicy, False),
        "AppCookieStickinessPolicy": (list, False),
        "AvailabilityZones": (list, False),
        "ConnectionDrainingPolicy": (ConnectionDrainingPolicy, False),
        "ConnectionSettings": (ConnectionSettings, False),
        "CrossZone": (boolean, False),
        "HealthCheck": (HealthCheck, False),
        "Instances": (list, False),
        "LBCookieStickinessPolicy": (list, False),
        "LoadBalancerName": (elb_name, False),
        "Listeners": (list, True),
        "Policies": (list, False),
        "Scheme": (str, False),
        "SecurityGroups": (list, False),
        "Subnets": (list, False),
        "Tags": ((Tags, list), False),
    }



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import (
    boolean,
    exactly_one,
    integer,
    mutually_exclusive,
    positive_integer,
)

KEY_ONLY = "KEY_ONLY"
VALUE_ONLY = "VALUE_ONLY"
KEY_AND_VALUE = "KEY_AND_VALUE"


class GitHubLocation(AWSProperty):
    props = {
        "CommitId": (str, True),
        "Repository": (str, True),
    }


class S3Location(AWSProperty):
    props = {
        "Bucket": (str, True),
        "BundleType": (str, True),
        "ETag": (str, False),
        "Key": (str, True),
        "Version": (str, False),
    }


class Revision(AWSProperty):
    props = {
        "GitHubLocation": (GitHubLocation, False),
        "RevisionType": (str, False),
        "S3Location": (S3Location, False),
    }


def deployment_option_validator(x):
    valid_values = ["WITH_TRAFFIC_CONTROL", "WITHOUT_TRAFFIC_CONTROL"]
    if x not in valid_values:
        raise ValueError(
            "Deployment Option value must be one of: %s" % ", ".join(valid_values)
        )
    return x


def deployment_type_validator(x):
    valid_values = ["IN_PLACE", "BLUE_GREEN"]
    if x not in valid_values:
        raise ValueError(
            "Deployment Type value must be one of: %s" % ", ".join(valid_values)
        )
    return x


class AutoRollbackConfiguration(AWSProperty):
    props = {"Enabled": (bool, False), "Events": ([str], False)}


class Deployment(AWSProperty):
    props = {
        "Description": (str, False),
        "IgnoreApplicationStopFailures": (bool, False),
        "Revision": (Revision, True),
    }


class DeploymentStyle(AWSProperty):
    props = {
        "DeploymentOption": (deployment_option_validator, False),
        "DeploymentType": (deployment_type_validator, False),
    }


class Ec2TagFilters(AWSProperty):
    props = {
        "Key": (str, False),
        "Type": (str, True),
        "Value": (str, False),
    }


class TagFilters(AWSProperty):
    props = {"Key": (str, False), "Type": (str, False), "Value": (str, False)}


class ElbInfoList(AWSProperty):
    props = {"Name": (str, False)}


class TargetGroupInfoList(AWSProperty):
    props = {"Name": (str, False)}


class LoadBalancerInfo(AWSProperty):
    props = {
        "ElbInfoList": ([ElbInfoList], False),
        "TargetGroupInfoList": ([TargetGroupInfoList], False),
    }

    def validate(self):
        conds = ["ElbInfoList", "TargetGroupInfoList"]
        exactly_one(self.__class__.__name__, self.properties, conds)


class OnPremisesInstanceTagFilters(AWSProperty):
    props = {
        "Key": (str, False),
        "Type": (str, False),
        "Value": (str, False),
    }


class Application(AWSObject):
    resource_type = "AWS::CodeDeploy::Application"

    props = {
        "ApplicationName": (str, False),
        "ComputePlatform": (str, False),
        "Tags": (Tags, False),
    }


class MinimumHealthyHosts(AWSProperty):
    props = {
        "Type": (str, True),
        "Value": (positive_integer, True),
    }


class TimeBasedCanary(AWSProperty):
    props = {
        "CanaryInterval": (integer, True),
        "CanaryPercentage": (integer, True),
    }


class TimeBasedLinear(AWSProperty):
    props = {
        "LinearInterval": (integer, True),
        "LinearPercentage": (integer, True),
    }


class TrafficRoutingConfig(AWSProperty):
    props = {
        "TimeBasedCanary": (TimeBasedCanary, False),
        "TimeBasedLinear": (TimeBasedLinear, False),
        "Type": (str, True),
    }


class DeploymentConfig(AWSObject):
    resource_type = "AWS::CodeDeploy::DeploymentConfig"

    props = {
        "ComputePlatform": (str, False),
        "DeploymentConfigName": (str, False),
        "MinimumHealthyHosts": (MinimumHealthyHosts, False),
        "TrafficRoutingConfig": (TrafficRoutingConfig, False),
    }


class Alarm(AWSProperty):
    props = {
        "Name": (str, False),
    }


class AlarmConfiguration(AWSProperty):
    props = {
        "Alarms": ([Alarm], False),
        "Enabled": (boolean, False),
        "IgnorePollAlarmFailure": (boolean, False),
    }


class BlueInstanceTerminationOption(AWSProperty):
    props = {
        "Action": (str, False),
        "TerminationWaitTimeInMinutes": (integer, False),
    }


class DeploymentReadyOption(AWSProperty):
    props = {
        "ActionOnTimeout": (str, False),
        "WaitTimeInMinutes": (integer, False),
    }


class GreenFleetProvisioningOption(AWSProperty):
    props = {
        "Action": (str, False),
    }


class BlueGreenDeploymentConfiguration(AWSProperty):
    props = {
        "DeploymentReadyOption": (DeploymentReadyOption, False),
        "GreenFleetProvisioningOption": (GreenFleetProvisioningOption, False),
        "TerminateBlueInstancesOnDeploymentSuccess": (
            BlueInstanceTerminationOption,
            False,
        ),
    }


class TriggerConfig(AWSProperty):
    props = {
        "TriggerEvents": ([str], False),
        "TriggerName": (str, False),
        "TriggerTargetArn": (str, False),
    }


class Ec2TagSetListObject(AWSProperty):
    props = {"Ec2TagGroup": ([Ec2TagFilters], False)}


class Ec2TagSet(AWSProperty):
    props = {"Ec2TagSetList": ([Ec2TagSetListObject], False)}


class ECSService(AWSProperty):
    props = {
        "ClusterName": (str, True),
        "ServiceName": (str, True),
    }


class OnPremisesTagSetObject(AWSProperty):
    props = {"OnPremisesTagGroup": ([TagFilters], False)}


class OnPremisesTagSetList(AWSProperty):
    props = {"OnPremisesTagSetList": ([OnPremisesTagSetObject], False)}


class OnPremisesTagSet(AWSProperty):
    props = {"OnPremisesTagSetList": (OnPremisesTagSetList, False)}


class DeploymentGroup(AWSObject):
    resource_type = "AWS::CodeDeploy::DeploymentGroup"

    props = {
        "AlarmConfiguration": (AlarmConfiguration, False),
        "ApplicationName": (str, True),
        "AutoRollbackConfiguration": (AutoRollbackConfiguration, False),
        "AutoScalingGroups": ([str], False),
        "BlueGreenDeploymentConfiguration": (BlueGreenDeploymentConfiguration, False),
        "Deployment": (Deployment, False),
        "DeploymentConfigName": (str, False),
        "DeploymentGroupName": (str, False),
        "DeploymentStyle": (DeploymentStyle, False),
        "ECSServices": ([ECSService], False),
        "Ec2TagFilters": ([Ec2TagFilters], False),
        "Ec2TagSet": (Ec2TagSet, False),
        "LoadBalancerInfo": (LoadBalancerInfo, False),
        "OnPremisesInstanceTagFilters": ([OnPremisesInstanceTagFilters], False),
        "OnPremisesInstanceTagSet": (OnPremisesTagSet, False),
        "ServiceRoleArn": (str, True),
        "TriggerConfigurations": ([TriggerConfig], False),
    }

    def validate(self):
        ec2_conds = ["EC2TagFilters", "Ec2TagSet"]
        onPremises_conds = ["OnPremisesInstanceTagFilters", "OnPremisesInstanceTagSet"]
        mutually_exclusive(self.__class__.__name__, self.properties, ec2_conds)
        mutually_exclusive(self.__class__.__name__, self.properties, onPremises_conds)



# Converted from Route53_RoundRobin.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from troposphere import Join, Parameter, Ref, Template
from troposphere.route53 import RecordSet, RecordSetGroup

t = Template()

t.set_description(
    "AWS CloudFormation Sample Template Route53_RoundRobin: Sample template "
    "showing how to use weighted round robin (WRR) DNS entried via Amazon "
    "Route 53. This contrived sample uses weighted CNAME records to "
    "illustrate that the weighting influences the return records. It assumes "
    " that you already have a Hosted Zone registered with Amazon Route 53. "
    "**WARNING** This template creates an Amazon EC2 instance. "
    "You will be billed for the AWS resources used if you create "
    "a stack from this template."
)

hostedzone = t.add_parameter(
    Parameter(
        "HostedZone",
        Description="The DNS name of an existing Amazon Route 53 hosted zone",
        Type="String",
    )
)

myDNSRecord = t.add_resource(
    RecordSetGroup(
        "myDNSRecord",
        HostedZoneName=Join("", [Ref(hostedzone), "."]),
        Comment="Contrived example to redirect to aws.amazon.com 75% of the time "
        "and www.amazon.com 25% of the time.",
        RecordSets=[
            RecordSet(
                SetIdentifier=Join(" ", [Ref("AWS::StackName"), "AWS"]),
                Name=Join(
                    "",
                    [
                        Ref("AWS::StackName"),
                        ".",
                        Ref("AWS::Region"),
                        ".",
                        Ref(hostedzone),
                        ".",
                    ],
                ),
                Type="CNAME",
                TTL="900",
                ResourceRecords=["aws.amazon.com"],
                Weight="3",
            ),
            RecordSet(
                SetIdentifier=Join(" ", [Ref("AWS::StackName"), "Amazon"]),
                Name=Join(
                    "",
                    [
                        Ref("AWS::StackName"),
                        ".",
                        Ref("AWS::Region"),
                        ".",
                        Ref(hostedzone),
                        ".",
                    ],
                ),
                Type="CNAME",
                TTL="900",
                ResourceRecords=["www.amazon.com"],
                Weight="1",
            ),
        ],
    )
)


print(t.to_json())



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty


class EksInfo(AWSProperty):
    props = {
        "Namespace": (str, True),
    }


class ContainerInfo(AWSProperty):
    props = {
        "EksInfo": (EksInfo, True),
    }


class ContainerProvider(AWSProperty):
    props = {
        "Id": (str, True),
        "Info": (ContainerInfo, True),
        "Type": (str, True),
    }


class VirtualCluster(AWSObject):
    resource_type = "AWS::EMRContainers::VirtualCluster"

    props = {
        "ContainerProvider": (ContainerProvider, True),
        "Name": (str, True),
        "Tags": (Tags, False),
    }



from . import AWSObject, AWSProperty
from .validators import boolean


class ParameterObjectAttribute(AWSProperty):
    props = {
        "Key": (str, True),
        "StringValue": (str, False),
    }


class ParameterObject(AWSProperty):
    props = {
        "Attributes": ([ParameterObjectAttribute], True),
        "Id": (str, True),
    }


class ParameterValue(AWSProperty):
    props = {
        "Id": (str, True),
        "StringValue": (str, True),
    }


class ObjectField(AWSProperty):
    props = {
        "Key": (str, True),
        "RefValue": (str, False),
        "StringValue": (str, False),
    }


class PipelineObject(AWSProperty):
    props = {
        "Fields": ([ObjectField], True),
        "Id": (str, True),
        "Name": (str, True),
    }


class PipelineTag(AWSProperty):
    props = {
        "Key": (str, True),
        "Value": (str, True),
    }


class Pipeline(AWSObject):
    resource_type = "AWS::DataPipeline::Pipeline"

    props = {
        "Activate": (boolean, False),
        "Description": (str, False),
        "Name": (str, True),
        "ParameterObjects": ([ParameterObject], False),
        "ParameterValues": ([ParameterValue], False),
        "PipelineObjects": ([PipelineObject], True),
        "PipelineTags": ([PipelineTag], False),
    }



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer


class ResourceSpec(AWSProperty):
    props = {
        "InstanceType": (str, False),
        "SageMakerImageArn": (str, False),
        "SageMakerImageVersionArn": (str, False),
    }


class App(AWSObject):
    resource_type = "AWS::SageMaker::App"

    props = {
        "AppName": (str, True),
        "AppType": (str, True),
        "DomainId": (str, True),
        "ResourceSpec": (ResourceSpec, False),
        "Tags": (Tags, False),
        "UserProfileName": (str, True),
    }


class FileSystemConfig(AWSProperty):
    props = {
        "DefaultGid": (integer, False),
        "DefaultUid": (integer, False),
        "MountPath": (str, False),
    }


class KernelSpec(AWSProperty):
    props = {
        "DisplayName": (str, False),
        "Name": (str, True),
    }


class KernelGatewayImageConfig(AWSProperty):
    props = {
        "FileSystemConfig": (FileSystemConfig, False),
        "KernelSpecs": ([KernelSpec], True),
    }


class AppImageConfig(AWSObject):
    resource_type = "AWS::SageMaker::AppImageConfig"

    props = {
        "AppImageConfigName": (str, True),
        "KernelGatewayImageConfig": (KernelGatewayImageConfig, False),
        "Tags": (Tags, False),
    }


class GitConfig(AWSProperty):
    props = {
        "Branch": (str, False),
        "RepositoryUrl": (str, True),
        "SecretArn": (str, False),
    }


class CodeRepository(AWSObject):
    resource_type = "AWS::SageMaker::CodeRepository"

    props = {
        "CodeRepositoryName": (str, False),
        "GitConfig": (GitConfig, True),
    }


class DataQualityAppSpecification(AWSProperty):
    props = {
        "ContainerArguments": ([str], False),
        "ContainerEntrypoint": ([str], False),
        "Environment": (dict, False),
        "ImageUri": (str, True),
        "PostAnalyticsProcessorSourceUri": (str, False),
        "RecordPreprocessorSourceUri": (str, False),
    }


class ConstraintsResource(AWSProperty):
    props = {
        "S3Uri": (str, False),
    }


class StatisticsResource(AWSProperty):
    props = {
        "S3Uri": (str, False),
    }


class DataQualityBaselineConfig(AWSProperty):
    props = {
        "BaseliningJobName": (str, False),
        "ConstraintsResource": (ConstraintsResource, False),
        "StatisticsResource": (StatisticsResource, False),
    }


class EndpointInput(AWSProperty):
    props = {
        "EndpointName": (str, True),
        "LocalPath": (str, True),
        "S3DataDistributionType": (str, False),
        "S3InputMode": (str, False),
    }


class DataQualityJobInput(AWSProperty):
    props = {
        "EndpointInput": (EndpointInput, True),
    }


class S3Output(AWSProperty):
    props = {
        "LocalPath": (str, True),
        "S3UploadMode": (str, False),
        "S3Uri": (str, True),
    }


class MonitoringOutput(AWSProperty):
    props = {
        "S3Output": (S3Output, True),
    }


class MonitoringOutputConfig(AWSProperty):
    props = {
        "KmsKeyId": (str, False),
        "MonitoringOutputs": ([MonitoringOutput], True),
    }


class ClusterConfig(AWSProperty):
    props = {
        "InstanceCount": (integer, True),
        "InstanceType": (str, True),
        "VolumeKmsKeyId": (str, False),
        "VolumeSizeInGB": (integer, True),
    }


class MonitoringResources(AWSProperty):
    props = {
        "ClusterConfig": (ClusterConfig, True),
    }


class VpcConfig(AWSProperty):
    props = {
        "Subnets": ([str], True),
        "SecurityGroupIds": ([str], True),
    }


class NetworkConfig(AWSProperty):
    props = {
        "EnableInterContainerTrafficEncryption": (boolean, False),
        "EnableNetworkIsolation": (boolean, False),
        "VpcConfig": (VpcConfig, False),
    }


class StoppingCondition(AWSProperty):
    props = {
        "MaxRuntimeInSeconds": (integer, True),
    }


class DataQualityJobDefinition(AWSObject):
    resource_type = "AWS::SageMaker::DataQualityJobDefinition"

    props = {
        "DataQualityAppSpecification": (DataQualityAppSpecification, True),
        "DataQualityBaselineConfig": (DataQualityBaselineConfig, False),
        "DataQualityJobInput": (DataQualityJobInput, True),
        "DataQualityJobOutputConfig": (MonitoringOutputConfig, True),
        "JobDefinitionName": (str, False),
        "JobResources": (MonitoringResources, True),
        "NetworkConfig": (NetworkConfig, False),
        "RoleArn": (str, True),
        "StoppingCondition": (StoppingCondition, False),
        "Tags": (Tags, False),
    }


class Device(AWSObject):
    resource_type = "AWS::SageMaker::Device"

    props = {
        "Device": (dict, False),
        "DeviceFleetName": (str, True),
        "Tags": (Tags, False),
    }


class EdgeOutputConfig(AWSProperty):
    props = {
        "KmsKeyId": (str, False),
        "S3OutputLocation": (str, True),
    }


class DeviceFleet(AWSObject):
    resource_type = "AWS::SageMaker::DeviceFleet"

    props = {
        "Description": (str, False),
        "DeviceFleetName": (str, True),
        "OutputConfig": (EdgeOutputConfig, True),
        "RoleArn": (str, True),
        "Tags": (Tags, False),
    }


class JupyterServerAppSettings(AWSProperty):
    props = {
        "DefaultResourceSpec": (ResourceSpec, False),
    }


class CustomImage(AWSProperty):
    props = {
        "AppImageConfigName": (str, True),
        "ImageName": (str, True),
        "ImageVersionNumber": (integer, False),
    }


class KernelGatewayAppSettings(AWSProperty):
    props = {
        "CustomImages": ([CustomImage], False),
        "DefaultResourceSpec": (ResourceSpec, False),
    }


class SharingSettings(AWSProperty):
    props = {
        "NotebookOutputOption": (str, False),
        "S3KmsKeyId": (str, False),
        "S3OutputPath": (str, False),
    }


class UserSettings(AWSProperty):
    props = {
        "ExecutionRole": (str, False),
        "JupyterServerAppSettings": (JupyterServerAppSettings, False),
        "KernelGatewayAppSettings": (KernelGatewayAppSettings, False),
        "SecurityGroups": ([str], False),
        "SharingSettings": (SharingSettings, False),
    }


class Domain(AWSObject):
    resource_type = "AWS::SageMaker::Domain"

    props = {
        "AppNetworkAccessType": (str, False),
        "AuthMode": (str, True),
        "DefaultUserSettings": (UserSettings, True),
        "DomainName": (str, True),
        "KmsKeyId": (str, False),
        "SubnetIds": ([str], True),
        "Tags": (Tags, False),
        "VpcId": (str, True),
    }


class Alarm(AWSProperty):
    props = {
        "AlarmName": (str, True),
    }


class AutoRollbackConfig(AWSProperty):
    props = {
        "Alarms": ([Alarm], True),
    }


class CapacitySize(AWSProperty):
    props = {
        "Type": (str, True),
        "Value": (integer, True),
    }


class TrafficRoutingConfig(AWSProperty):
    props = {
        "CanarySize": (CapacitySize, False),
        "Type": (str, True),
        "WaitIntervalInSeconds": (integer, False),
    }


class BlueGreenUpdatePolicy(AWSProperty):
    props = {
        "MaximumExecutionTimeoutInSeconds": (integer, False),
        "TerminationWaitInSeconds": (integer, False),
        "TrafficRoutingConfiguration": (TrafficRoutingConfig, True),
    }


class DeploymentConfig(AWSProperty):
    props = {
        "AutoRollbackConfiguration": (AutoRollbackConfig, False),
        "BlueGreenUpdatePolicy": (BlueGreenUpdatePolicy, True),
    }


class VariantProperty(AWSProperty):
    props = {
        "VariantPropertyType": (str, False),
    }


class Endpoint(AWSObject):
    resource_type = "AWS::SageMaker::Endpoint"

    props = {
        "DeploymentConfig": (DeploymentConfig, False),
        "EndpointConfigName": (str, True),
        "EndpointName": (str, False),
        "ExcludeRetainedVariantProperties": ([VariantProperty], False),
        "RetainAllVariantProperties": (boolean, False),
        "Tags": (Tags, False),
    }


class AsyncInferenceClientConfig(AWSProperty):
    props = {
        "MaxConcurrentInvocationsPerInstance": (integer, False),
    }


class AsyncInferenceNotificationConfig(AWSProperty):
    props = {
        "ErrorTopic": (str, False),
        "SuccessTopic": (str, False),
    }


class AsyncInferenceOutputConfig(AWSProperty):
    props = {
        "KmsKeyId": (str, False),
        "NotificationConfig": (AsyncInferenceNotificationConfig, False),
        "S3OutputPath": (str, True),
    }


class AsyncInferenceConfig(AWSProperty):
    props = {
        "ClientConfig": (AsyncInferenceClientConfig, False),
        "OutputConfig": (AsyncInferenceOutputConfig, True),
    }


class CaptureContentTypeHeader(AWSProperty):
    props = {
        "CsvContentTypes": ([str], False),
        "JsonContentTypes": ([str], False),
    }


class CaptureOption(AWSProperty):
    props = {
        "CaptureMode": (str, True),
    }


class DataCaptureConfig(AWSProperty):
    props = {
        "CaptureContentTypeHeader": (CaptureContentTypeHeader, False),
        "CaptureOptions": ([CaptureOption], True),
        "DestinationS3Uri": (str, True),
        "EnableCapture": (boolean, False),
        "InitialSamplingPercentage": (integer, True),
        "KmsKeyId": (str, False),
    }


class ProductionVariant(AWSProperty):
    props = {
        "ModelName": (str, True),
        "VariantName": (str, True),
        "InitialInstanceCount": (integer, True),
        "InstanceType": (str, True),
        "InitialVariantWeight": (float, True),
    }


class EndpointConfig(AWSObject):
    resource_type = "AWS::SageMaker::EndpointConfig"

    props = {
        "AsyncInferenceConfig": (AsyncInferenceConfig, False),
        "DataCaptureConfig": (DataCaptureConfig, False),
        "EndpointConfigName": (str, False),
        "KmsKeyId": (str, False),
        "ProductionVariants": ([ProductionVariant], True),
        "Tags": (Tags, False),
    }


class FeatureDefinition(AWSProperty):
    props = {
        "FeatureName": (str, True),
        "FeatureType": (str, True),
    }


class FeatureGroup(AWSObject):
    resource_type = "AWS::SageMaker::FeatureGroup"

    props = {
        "Description": (str, False),
        "EventTimeFeatureName": (str, True),
        "FeatureDefinitions": ([FeatureDefinition], True),
        "FeatureGroupName": (str, True),
        "OfflineStoreConfig": (dict, False),
        "OnlineStoreConfig": (dict, False),
        "RecordIdentifierFeatureName": (str, True),
        "RoleArn": (str, False),
        "Tags": (Tags, False),
    }


class Image(AWSObject):
    resource_type = "AWS::SageMaker::Image"

    props = {
        "ImageDescription": (str, False),
        "ImageDisplayName": (str, False),
        "ImageName": (str, True),
        "ImageRoleArn": (str, True),
        "Tags": (Tags, False),
    }


class ImageVersion(AWSObject):
    resource_type = "AWS::SageMaker::ImageVersion"

    props = {
        "BaseImage": (str, True),
        "ImageName": (str, True),
    }


class ImageConfig(AWSProperty):
    props = {
        "RepositoryAccessMode": (str, True),
    }


class MultiModelConfig(AWSProperty):
    props = {
        "ModelCacheSetting": (str, False),
    }


class ContainerDefinition(AWSProperty):
    props = {
        "ContainerHostname": (str, False),
        "Environment": (dict, False),
        "Image": (str, False),
        "ImageConfig": (ImageConfig, False),
        "Mode": (str, False),
        "ModelDataUrl": (str, False),
        "ModelPackageName": (str, False),
        "MultiModelConfig": (MultiModelConfig, False),
    }


class InferenceExecutionConfig(AWSProperty):
    props = {
        "Mode": (str, True),
    }


class Model(AWSObject):
    resource_type = "AWS::SageMaker::Model"

    props = {
        "Containers": ([ContainerDefinition], False),
        "EnableNetworkIsolation": (boolean, False),
        "ExecutionRoleArn": (str, True),
        "InferenceExecutionConfig": (InferenceExecutionConfig, False),
        "ModelName": (str, False),
        "PrimaryContainer": (ContainerDefinition, False),
        "Tags": (Tags, False),
        "VpcConfig": (VpcConfig, False),
    }


class ModelBiasAppSpecification(AWSProperty):
    props = {
        "ConfigUri": (str, True),
        "Environment": (dict, False),
        "ImageUri": (str, True),
    }


class ModelBiasBaselineConfig(AWSProperty):
    props = {
        "BaseliningJobName": (str, False),
        "ConstraintsResource": (ConstraintsResource, False),
    }


class MonitoringGroundTruthS3Input(AWSProperty):
    props = {
        "S3Uri": (str, True),
    }


class ModelBiasJobInput(AWSProperty):
    props = {
        "EndpointInput": (EndpointInput, True),
        "GroundTruthS3Input": (MonitoringGroundTruthS3Input, True),
    }


class ModelBiasJobDefinition(AWSObject):
    resource_type = "AWS::SageMaker::ModelBiasJobDefinition"

    props = {
        "JobDefinitionName": (str, False),
        "JobResources": (MonitoringResources, True),
        "ModelBiasAppSpecification": (ModelBiasAppSpecification, True),
        "ModelBiasBaselineConfig": (ModelBiasBaselineConfig, False),
        "ModelBiasJobInput": (ModelBiasJobInput, True),
        "ModelBiasJobOutputConfig": (MonitoringOutputConfig, True),
        "NetworkConfig": (NetworkConfig, False),
        "RoleArn": (str, True),
        "StoppingCondition": (StoppingCondition, False),
        "Tags": (Tags, False),
    }


class ModelExplainabilityAppSpecification(AWSProperty):
    props = {
        "ConfigUri": (str, True),
        "Environment": (dict, False),
        "ImageUri": (str, True),
    }


class ModelExplainabilityBaselineConfig(AWSProperty):
    props = {
        "BaseliningJobName": (str, False),
        "ConstraintsResource": (ConstraintsResource, False),
    }


class ModelExplainabilityJobInput(AWSProperty):
    props = {
        "EndpointInput": (EndpointInput, True),
    }


class ModelExplainabilityJobDefinition(AWSObject):
    resource_type = "AWS::SageMaker::ModelExplainabilityJobDefinition"

    props = {
        "JobDefinitionName": (str, False),
        "JobResources": (MonitoringResources, True),
        "ModelExplainabilityAppSpecification": (
            ModelExplainabilityAppSpecification,
            True,
        ),
        "ModelExplainabilityBaselineConfig": (ModelExplainabilityBaselineConfig, False),
        "ModelExplainabilityJobInput": (ModelExplainabilityJobInput, True),
        "ModelExplainabilityJobOutputConfig": (MonitoringOutputConfig, True),
        "NetworkConfig": (NetworkConfig, False),
        "RoleArn": (str, True),
        "StoppingCondition": (StoppingCondition, False),
        "Tags": (Tags, False),
    }


class ModelPackageGroup(AWSObject):
    resource_type = "AWS::SageMaker::ModelPackageGroup"

    props = {
        "ModelPackageGroupDescription": (str, False),
        "ModelPackageGroupName": (str, True),
        "ModelPackageGroupPolicy": (dict, False),
        "Tags": (Tags, False),
    }


class ModelQualityAppSpecification(AWSProperty):
    props = {
        "ContainerArguments": ([str], False),
        "ContainerEntrypoint": ([str], False),
        "Environment": (dict, False),
        "ImageUri": (str, True),
        "PostAnalyticsProcessorSourceUri": (str, False),
        "ProblemType": (str, True),
        "RecordPreprocessorSourceUri": (str, False),
    }


class ModelQualityBaselineConfig(AWSProperty):
    props = {
        "BaseliningJobName": (str, False),
        "ConstraintsResource": (ConstraintsResource, False),
    }


class ModelQualityJobInput(AWSProperty):
    props = {
        "EndpointInput": (EndpointInput, True),
        "GroundTruthS3Input": (MonitoringGroundTruthS3Input, True),
    }


class ModelQualityJobDefinition(AWSObject):
    resource_type = "AWS::SageMaker::ModelQualityJobDefinition"

    props = {
        "JobDefinitionName": (str, False),
        "JobResources": (MonitoringResources, True),
        "ModelQualityAppSpecification": (ModelQualityAppSpecification, True),
        "ModelQualityBaselineConfig": (ModelQualityBaselineConfig, False),
        "ModelQualityJobInput": (ModelQualityJobInput, True),
        "ModelQualityJobOutputConfig": (MonitoringOutputConfig, True),
        "NetworkConfig": (NetworkConfig, False),
        "RoleArn": (str, True),
        "StoppingCondition": (StoppingCondition, False),
        "Tags": (Tags, False),
    }


class MonitoringExecutionSummary(AWSProperty):
    props = {
        "CreationTime": (str, True),
        "EndpointName": (str, False),
        "FailureReason": (str, False),
        "LastModifiedTime": (str, True),
        "MonitoringExecutionStatus": (str, True),
        "MonitoringScheduleName": (str, True),
        "ProcessingJobArn": (str, False),
        "ScheduledTime": (str, True),
    }


class BaselineConfig(AWSProperty):
    props = {
        "ConstraintsResource": (ConstraintsResource, False),
        "StatisticsResource": (StatisticsResource, False),
    }


class MonitoringAppSpecification(AWSProperty):
    props = {
        "ContainerArguments": ([str], False),
        "ContainerEntrypoint": ([str], False),
        "ImageUri": (str, True),
        "PostAnalyticsProcessorSourceUri": (str, False),
        "RecordPreprocessorSourceUri": (str, False),
    }


class MonitoringInput(AWSProperty):
    props = {
        "EndpointInput": (EndpointInput, True),
    }


class MonitoringInputs(AWSProperty):
    props = {
        "MonitoringInputs": ([MonitoringInput], False),
    }


class MonitoringJobDefinition(AWSProperty):
    props = {
        "BaselineConfig": (BaselineConfig, False),
        "Environment": (dict, False),
        "MonitoringAppSpecification": (MonitoringAppSpecification, True),
        "MonitoringInputs": (MonitoringInputs, True),
        "MonitoringOutputConfig": (MonitoringOutputConfig, True),
        "MonitoringResources": (MonitoringResources, True),
        "NetworkConfig": (NetworkConfig, False),
        "RoleArn": (str, True),
        "StoppingCondition": (StoppingCondition, False),
    }


class ScheduleConfig(AWSProperty):
    props = {
        "ScheduleExpression": (str, True),
    }


class MonitoringScheduleConfig(AWSProperty):
    props = {
        "MonitoringJobDefinition": (MonitoringJobDefinition, True),
        "ScheduleConfig": (ScheduleConfig, False),
    }


class MonitoringSchedule(AWSObject):
    resource_type = "AWS::SageMaker::MonitoringSchedule"

    props = {
        "CreationTime": (str, False),
        "EndpointName": (str, False),
        "FailureReason": (str, False),
        "LastModifiedTime": (str, False),
        "LastMonitoringExecutionSummary": (MonitoringExecutionSummary, False),
        "MonitoringScheduleArn": (str, False),
        "MonitoringScheduleConfig": (MonitoringScheduleConfig, True),
        "MonitoringScheduleName": (str, True),
        "MonitoringScheduleStatus": (str, False),
        "Tags": (Tags, False),
    }


class NotebookInstance(AWSObject):
    resource_type = "AWS::SageMaker::NotebookInstance"

    props = {
        "AcceleratorTypes": ([str], False),
        "AdditionalCodeRepositories": ([str], False),
        "DefaultCodeRepository": (str, False),
        "DirectInternetAccess": (str, False),
        "InstanceType": (str, True),
        "KmsKeyId": (str, False),
        "LifecycleConfigName": (str, False),
        "NotebookInstanceName": (str, False),
        "RoleArn": (str, True),
        "RootAccess": (str, False),
        "SecurityGroupIds": ([str], False),
        "SubnetId": (str, False),
        "Tags": (Tags, False),
        "VolumeSizeInGB": (integer, False),
    }


class NotebookInstanceLifecycleHook(AWSProperty):
    props = {
        "Content": (str, False),
    }


class NotebookInstanceLifecycleConfig(AWSObject):
    resource_type = "AWS::SageMaker::NotebookInstanceLifecycleConfig"

    props = {
        "NotebookInstanceLifecycleConfigName": (str, False),
        "OnCreate": ([NotebookInstanceLifecycleHook], False),
        "OnStart": ([NotebookInstanceLifecycleHook], False),
    }


class Pipeline(AWSObject):
    resource_type = "AWS::SageMaker::Pipeline"

    props = {
        "PipelineDefinition": (dict, True),
        "PipelineDescription": (str, False),
        "PipelineDisplayName": (str, False),
        "PipelineName": (str, True),
        "RoleArn": (str, True),
        "Tags": (Tags, False),
    }


class Project(AWSObject):
    resource_type = "AWS::SageMaker::Project"

    props = {
        "ProjectDescription": (str, False),
        "ProjectName": (str, True),
        "ServiceCatalogProvisioningDetails": (dict, True),
        "Tags": (Tags, False),
    }


class UserProfile(AWSObject):
    resource_type = "AWS::SageMaker::UserProfile"

    props = {
        "DomainId": (str, True),
        "SingleSignOnUserIdentifier": (str, False),
        "SingleSignOnUserValue": (str, False),
        "Tags": (Tags, False),
        "UserProfileName": (str, True),
        "UserSettings": (UserSettings, False),
    }


class CognitoMemberDefinition(AWSProperty):
    props = {
        "CognitoClientId": (str, True),
        "CognitoUserGroup": (str, True),
        "CognitoUserPool": (str, True),
    }


class MemberDefinition(AWSProperty):
    props = {
        "CognitoMemberDefinition": (CognitoMemberDefinition, True),
    }


class NotificationConfiguration(AWSProperty):
    props = {
        "NotificationTopicArn": (str, True),
    }


class Workteam(AWSObject):
    resource_type = "AWS::SageMaker::Workteam"

    props = {
        "Description": (str, False),
        "MemberDefinitions": ([MemberDefinition], False),
        "NotificationConfiguration": (NotificationConfiguration, False),
        "Tags": (Tags, False),
        "WorkteamName": (str, False),
    }
